"""
市值指数
"""
import pandas as pd
import numpy as np
import hbshare as hbs
import os
from tqdm import tqdm
from datetime import datetime
from hbshare.fe.common.util.data_loader import get_trading_day_list
from hbshare.fe.common.util.config import style_name, industry_name
from hbshare.fe.common.util.config import factor_map_dict
from WindPy import w

w.start()


class CapIndex:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date

    @staticmethod
    def get_mkt_info(start_date, end_date):
        path = r'D:\kevin\risk_model_jy\RiskModel\data\common_data\chg_pct'
        listdir = os.listdir(path)
        listdir = [x for x in listdir if start_date <= x.split('.')[0] <= end_date]
        data = []
        for filename in listdir:
            date_t_rate = pd.read_csv(os.path.join(path, filename), dtype={"tradeDate": str})
            date_t_rate['ticker'] = date_t_rate['ticker'].apply(lambda x: str(x).zfill(6))
            data.append(date_t_rate)

        data = pd.concat(data)
        # filter
        data = data[(data['dailyReturnReinv'] >= -0.2) & (data['dailyReturnReinv'] <= 0.2)]
        data = pd.pivot_table(data, index='tradeDate', columns='ticker', values='dailyReturnReinv').sort_index()
        data = data.dropna(how='any', axis=1)

        return data

    @staticmethod
    def _load_benchmark_components(date):
        sql_script = "SELECT * FROM hsjy_gg.SecuMain where SecuCategory = 4 and " \
                     "SecuCode in ('000300', '000905', '000852', '000985')"
        res = hbs.db_data_query('readonly', sql_script)
        index_info = pd.DataFrame(res['data'])
        inner_code_series = index_info.set_index('SECUCODE')['INNERCODE']

        weight = []
        for benchmark_id in ['000300', '000905', '000852', '000985']:
            inner_code = inner_code_series.loc[benchmark_id]
            sql_script = "SELECT (select a.SecuCode from hsjy_gg.SecuMain a where a.InnerCode = b.InnerCode and " \
                         "rownum = 1) SecuCode, b.EndDate, b.Weight FROM hsjy_gg.LC_IndexComponentsWeight b WHERE " \
                         "b.IndexCode = '{}' and b.EndDate = to_date('{}', 'yyyymmdd')".format(inner_code, date)
            data = pd.DataFrame(hbs.db_data_query('readonly', sql_script, page_size=5000)['data'])
            weight_df = data.rename(
                columns={"SECUCODE": "ticker", "ENDDATE": "effDate", "WEIGHT": "weight"})
            weight_df['benchmark_id'] = benchmark_id
            weight.append(weight_df[['ticker', 'benchmark_id']])

        return pd.concat(weight)

    @staticmethod
    def _load_style_exposure(date):
        sql_script = "SELECT * FROM st_ashare.r_st_barra_style_factor where TRADE_DATE = '{}'".format(date)
        res = hbs.db_data_query('alluser', sql_script, page_size=5000)
        exposure_df = pd.DataFrame(res['data']).set_index('ticker')
        ind_names = [x.lower() for x in industry_name['sw'].values()]
        exposure_df = exposure_df[style_name + ind_names]

        return exposure_df

    def calculate_cap_index(self):
        month_list = get_trading_day_list(self.start_date, self.end_date, frequency="month")
        month_list.append(self.end_date)
        reverse_ind = dict([(value.lower(), key) for (key, value) in industry_name['sw'].items()])
        ret_list = []
        style_dict = {}
        industry_dict = {}
        ret_attr = {}
        for i in tqdm(range(1, len(month_list))):
            p_date, t_date = month_list[i - 1], month_list[i]
            period_data = self.get_mkt_info(p_date, t_date).fillna(0.)
            bm_components = self._load_benchmark_components(p_date)
            mkv_info = pd.read_json(
                r'D:\kevin\risk_model_jy\RiskModel\data\common_data/market_value/%s.json' % p_date,
                dtype={'ticker': str})
            mkv_info = mkv_info[(mkv_info['ticker'].str[0].isin(['0', '3', '6']))].set_index(
                'ticker')[['marketValue', 'negMarketValue']]
            style_exposure_df = self._load_style_exposure(p_date)

            idx = set(period_data.columns).intersection(set(bm_components['ticker'].unique())).intersection(
                set(mkv_info.index)).intersection(set(style_exposure_df.index))
            # 指数收益
            period_data = period_data.T.reindex(idx).T
            mkv_info = mkv_info.reindex(idx)
            bm_components = bm_components.merge(mkv_info, left_on='ticker', right_index=True).sort_values(
                by=['ticker', 'benchmark_id']).drop_duplicates(subset=['ticker'], keep='first')
            # tmp = bm_components
            bm_components = pd.pivot_table(
                bm_components, index='ticker', columns='benchmark_id', values='negMarketValue').fillna(0.) / 1e+9
            bm_components = (bm_components / bm_components.sum()).reindex(idx)
            index_ret = (1 + period_data).cumprod().dot(bm_components).pct_change().dropna()
            ret_list.append(index_ret)
            # 指数风格
            # style_exposure_df = style_exposure_df.reindex(idx)
            # index_style = bm_components.T.dot(style_exposure_df.astype(float)).T
            #
            # style_df = index_style.loc[style_name]
            # style_df.index = style_df.index.map(factor_map_dict)
            # benchmark_id_map = {"000300": "沪深300", "000905": "中证500", "000852": "中证1000", "000985": "剩余小票"}
            # style_df = style_df.rename(columns=benchmark_id_map)
            # style_dict[p_date] = style_df
            #
            # ind_df = index_style[10:]
            # ind_df.index = [reverse_ind[x] for x in ind_df.index]
            # ind_df = ind_df.rename(columns=benchmark_id_map)
            # industry_dict[p_date] = ind_df
            # # 风格拆解： 风格 + 行业
            # sql_script = "SELECT * FROM st_ashare.r_st_barra_factor_return where " \
            #              "TRADE_DATE > '{}' and TRADE_DATE <= {}".format(p_date, t_date)
            # res = hbs.db_data_query('alluser', sql_script, page_size=5000)
            # factor_return = pd.DataFrame(res['data'])
            # factor_return = pd.pivot_table(
            #     factor_return, index='trade_date', columns='factor_name', values='factor_ret').sort_index()
            # factor_return = (1 + factor_return).prod() - 1
            # factor_return.index = factor_return.index.str.lower()
            # index_style.loc["country"] = 1.
            #
            # idx = set(factor_return.index).intersection(set(index_style.index))
            #
            # f_ret = index_style.reindex(idx).mul(np.array(factor_return.reindex(idx)).reshape(-1, 1))
            # f_ret.loc['spec'] = (1 + index_ret).prod() - 1 - f_ret.sum()
            #
            # ret_attr[t_date] = f_ret

            # mkv_des = tmp.groupby('benchmark_id')['marketValue'].describe()

        ret_df = pd.concat(ret_list).sort_index()
        ret_df.loc[self.start_date] = 0.
        ret_df = ret_df.sort_index()

        # 风格归因的累计
        attr_list = []
        for date in sorted(ret_attr.keys()):
            tmp = ret_attr[date].copy()
            tmp['factor'] = tmp.index
            tmp['trade_date'] = date
            tmp.reset_index(drop=True, inplace=True)
            attr_list.append(tmp)

        attr_df = pd.concat(attr_list)

        # 沪深300的拆解
        sub_attr = []
        for col in ['000300', '000905', '000852', '000985']:
            sub_df = pd.pivot_table(attr_df, index='trade_date', columns='factor', values=col).sort_index()
            index_return = sub_df.sum(axis=1)
            kt = np.log(1 + index_return) / index_return
            r = (1 + index_return).prod() - 1
            k = np.log(1 + r) / r
            cum_attr = sub_df.T.multiply(kt / k).sum(axis=1).to_frame(col)
            sub_attr.append(cum_attr)

        sub_attr = pd.concat(sub_attr, axis=1)
        sub_attr_style = sub_attr.loc[style_name]
        sub_attr_style.index = sub_attr_style.index.map(factor_map_dict)
        ind_name = [x for x in sub_attr.index.tolist() if x not in style_name]
        ind_name = [x for x in ind_name if x not in ['spec', 'country']]
        sub_attr_style.loc['行业'] = sub_attr.loc[ind_name].sum()
        sub_attr_style.loc['国家因子'] = sub_attr.loc['country']
        sub_attr_style.loc['特质性收益'] = sub_attr.loc['spec']

        sub_attr_ind = sub_attr.loc[ind_name]
        sub_attr_ind.index = [reverse_ind[x] for x in sub_attr_ind.index]

        return ret_df


class SmallCapIndex:
    """
    衍复的小市值指增基准的一些测试
    """
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date

    @staticmethod
    def cal_annual_return(return_series):
        T = len(return_series)
        annual_return = (1 + return_series).prod() ** (252 / T) - 1

        return annual_return

    @staticmethod
    def cal_annual_volatility(return_series):
        vol = return_series.std() * np.sqrt(252)

        return vol

    @staticmethod
    def cal_max_drawdown(nav_series):
        drawdown_series = nav_series / (nav_series.cummax()) - 1

        return drawdown_series.min()

    @staticmethod
    def cal_sharpe_ratio(return_series, rf):
        annual_return = cal_annual_return(return_series)
        vol = cal_annual_volatility(return_series)
        sharpe_ratio = (annual_return - rf) / vol

        return sharpe_ratio

    def run(self):
        path = "D:\\MarketInfoSaver"
        listdir = os.listdir(path)
        listdir = [x for x in listdir if self.start_date <= x.split('_')[-1].split('.')[0] <= self.end_date]
        # Wind小市值指数:
        month_end = get_trading_day_list(self.start_date, self.end_date, frequency="month")
        month_end.append(self.end_date)
        ret_list1 = []
        ret_list2 = []
        universe_series = []
        other_series = []
        tmp1_list = []
        tmp2_list = []
        neg_list = []
        for i in tqdm(range(1, len(month_end))):
            pre_date, t_date = month_end[i - 1], month_end[i]
            sub_dir = [x for x in listdir if pre_date <= x.split('_')[-1].split('.')[0] <= t_date]
            mkt_list = []
            for filename in sub_dir:
                trade_date = filename.split('.')[0].split('_')[-1]
                date_t_data = pd.read_csv(os.path.join(path, filename))
                date_t_data['ticker'] = date_t_data['ticker'].apply(lambda x: str(x).zfill(6))
                date_t_data.loc[date_t_data['turnoverValue'] < 1e-8, 'dailyReturnReinv'] = np.NaN
                date_t_data['trade_date'] = trade_date
                mkt_list.append(date_t_data)
            mkt_info = pd.concat(mkt_list)
            mkt_info['dailyReturnReinv'] /= 100.
            mkt_info['dailyReturnReinv'] = mkt_info['dailyReturnReinv'].fillna(0.)
            universe_list = \
                mkt_info[(mkt_info['trade_date'] == pre_date) & (mkt_info['marketValue'] < 1e+10)]['ticker'].tolist()

            tmp1 = mkt_info[(mkt_info['trade_date'] == pre_date) & (mkt_info['marketValue'] < 1e+10)]['negMarketValue'].sum()
            tmp2 = mkt_info[mkt_info['trade_date'] == pre_date]['negMarketValue'].sum()
            tmp = tmp1 / tmp2
            tmp1_list.append(tmp1)
            tmp2_list.append(tmp2)
            neg_list.append(tmp)
            universe_series.append(len(universe_list))
            other_series.append(len(mkt_info[(mkt_info['trade_date'] == pre_date)]))

            pivot_df = pd.pivot_table(mkt_info, index='trade_date', columns='ticker',
                                      values='dailyReturnReinv').sort_index().fillna(0.)[universe_list]
            # 每天再平衡, Wind采用的也是这个模式
            period_ret1 = pivot_df.mean(axis=1)[1:]
            # 不做再平衡
            period_ret2 = (1 + pivot_df).cumprod().mean(axis=1).pct_change().dropna()
            ret_list1.append(period_ret1)
            ret_list2.append(period_ret2)

        ret_series1 = pd.concat(ret_list1).sort_index()
        nav_series1 = (1 + ret_series1).cumprod()
        nav_series1.loc[self.start_date] = 1.
        nav_series1 = nav_series1.sort_index()
        ret_series2 = pd.concat(ret_list2).sort_index()
        nav_series2 = (1 + ret_series2).cumprod()
        nav_series2.loc[self.start_date] = 1.
        nav_series2 = nav_series2.sort_index()

        count_df = pd.DataFrame(
            {"trade_date": month_end[:-1], "universe_num": universe_series, "other_num": other_series,
             "all_neg": tmp2_list, "small_neg": tmp1_list,  "neg_ratio": neg_list}).sort_index()
        count_df['other_num'] -= 1800
        count_df = count_df.set_index('trade_date')

        # Wind指数对比
        res = w.wsd("8841425.WI,000852.SH", "close", "2016-12-30", "2023-02-16", "")
        date_list = [datetime.strftime(x, "%Y%m%d") for x in res.Times]
        w_data = pd.DataFrame(res.Data, index=res.Codes, columns=date_list).T
        w_data /= w_data.iloc[0]

        compare_df = nav_series1.to_frame('每日再平衡').merge(
            w_data, left_index=True, right_index=True).merge(
            nav_series2.to_frame('月度复利'), left_index=True, right_index=True)
        compare_df.rename(columns={"8841425.WI": "万得小市值指数", "000852.SH": "中证1000"}, inplace=True)

        nav_df = compare_df.copy()
        portfolio_index_df = pd.DataFrame(
            index=nav_df.columns, columns=['年化收益', '年化波动', '最大回撤', 'Sharpe', '胜率', '平均损益比'])
        portfolio_index_df.loc[:, '年化收益'] = nav_df.pct_change().dropna(how='all').apply(self.cal_annual_return, axis=0)
        portfolio_index_df.loc[:, '年化波动'] = \
            nav_df.pct_change().dropna(how='all').apply(self.cal_annual_volatility, axis=0)
        portfolio_index_df.loc[:, '最大回撤'] = nav_df.apply(self.cal_max_drawdown, axis=0)
        portfolio_index_df.loc[:, 'Sharpe'] = \
            nav_df.pct_change().dropna(how='all').apply(lambda x: self.cal_sharpe_ratio(x, 0.015), axis=0)
        portfolio_index_df.loc[:, '胜率'] = \
            nav_df.pct_change().dropna(how='all').apply(lambda x: x.gt(0).sum() / len(x), axis=0)
        portfolio_index_df.loc[:, '平均损益比'] = \
            nav_df.pct_change().dropna(how='all').apply(lambda x: x[x > 0].mean() / x[x < 0].abs().mean(), axis=0)
        portfolio_index_df.index.name = '产品名称'
        portfolio_index_df.reset_index(inplace=True)
        # 格式处理
        portfolio_index_df['年化收益'] = portfolio_index_df['年化收益'].apply(lambda x: format(x, '.2%'))
        portfolio_index_df['年化波动'] = portfolio_index_df['年化波动'].apply(lambda x: format(x, '.2%'))
        portfolio_index_df['最大回撤'] = portfolio_index_df['最大回撤'].apply(lambda x: format(x, '.2%'))
        portfolio_index_df['Sharpe'] = portfolio_index_df['Sharpe'].round(2)
        portfolio_index_df['胜率'] = portfolio_index_df['胜率'].apply(lambda x: format(x, '.1%'))
        portfolio_index_df['平均损益比'] = portfolio_index_df['平均损益比'].round(2)

        return compare_df


class EquityWeightedMarketIndex:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date

    @staticmethod
    def _calc_group_weight(factor_series, group_num=5):
        n = factor_series.shape[0]
        p_list = [round((n / group_num) * i, 1) for i in range(0, group_num + 1)]
        group_df = pd.DataFrame(index=factor_series.index, columns=['group_{}'.format(i) for i in range(1, group_num + 1)])
        group_df = pd.merge(
            factor_series.to_frame('factor'), group_df, left_index=True, right_index=True).sort_values(by='factor')

        for i in range(len(p_list) - 1):
            point1, point2 = p_list[i], p_list[i + 1]
            tmp1 = group_df.index[np.floor(point1).astype(int)]
            if i == len(p_list) - 2:
                tmp2 = group_df.index[np.floor(point2).astype(int) - 1]
            else:
                tmp2 = group_df.index[np.floor(point2).astype(int)]
            group_df.loc[tmp1: tmp2, 'group_{}'.format(i + 1)] = 1.

        for i in range(1, len(p_list) - 1):
            point = p_list[i]
            tmp = group_df.index[np.floor(point).astype(int)]
            t_values = round(point - int(point), 2)
            group_df.loc[tmp, 'group_{}'.format(i)] = t_values
            group_df.loc[tmp, 'group_{}'.format(i + 1)] = round(1 - t_values, 1)

        group_df = group_df.fillna(0.)

        return group_df

    def run(self):
        path = "D:\\MarketInfoSaver"
        listdir = os.listdir(path)
        listdir = [x for x in listdir if self.start_date < x.split('_')[-1].split('.')[0] <= self.end_date]
        ret_list = []
        for filename in tqdm(listdir):
            trade_date = filename.split('.')[0].split('_')[-1]
            date_t_data = pd.read_csv(os.path.join(path, filename))
            date_t_data['ticker'] = date_t_data['ticker'].apply(lambda x: str(x).zfill(6))
            date_t_data['trade_date'] = trade_date
            date_t_data.loc[date_t_data['turnoverValue'] < 1e-8, 'dailyReturnReinv'] = np.NaN
            date_t_data = date_t_data.dropna().set_index('ticker')
            date_t_data = date_t_data[date_t_data['marketValue'] >= date_t_data['marketValue'].quantile(0.1)]
            group_df = self._calc_group_weight(date_t_data['marketValue'], group_num=5)
            group_df = group_df[group_df.columns[1:]]
            group_df /= group_df.sum()

            group_ret = group_df.reindex(date_t_data.index).T.dot(date_t_data['dailyReturnReinv']) / 100.

            ret_list.append(group_ret.to_frame(trade_date))

        ret_df = pd.concat(ret_list, axis=1).T.sort_index()
        nav_df = (1 + ret_df).cumprod()
        nav_df.loc[self.start_date] = 1.
        nav_df = nav_df.sort_index()

        return nav_df


if __name__ == '__main__':
    # CapIndex('20221230', '20230210').calculate_cap_index()
    # EquityWeightedMarketIndex('20211231', '20221230').run()
    SmallCapIndex('20161230', '20230216').run()


"""
四象限风格轮动-专题报告
"""
import pandas as pd
import numpy as np
import hbshare as hbs
from datetime import datetime
from sqlalchemy import create_engine
from hbshare.quant.Kevin.rm_associated.config import engine_params, style_names
from hbshare.fe.common.util.data_loader import get_trading_day_list
import statsmodels.api as sm

map_dict = {
    "399373": "大盘价值",
    "399372": "大盘成长",
    "399376": "小盘成长",
    "399377": "小盘价值",
    "000300": "沪深300",
    "000905": "中证500",
    "000852": "中证1000"
}


class StyleRotation:
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date
        self._load_data()

    def _load_style_index(self):
        index_codes = ['399372', '399373', '399376', '399377', '000300', '000905', '000852']
        index_list = []
        for index_code in index_codes:
            sql_script = "SELECT jyrq as TRADEDATE, zqmc as INDEXNAME, zqdm as TICKER, spjg as TCLOSE from " \
                         "st_market.t_st_zs_hqql WHERE " \
                         "ZQDM = '{}' and JYRQ >= {} and " \
                         "JYRQ <= {}".format(index_code, self.start_date, self.end_date)
            res = hbs.db_data_query("alluser", sql_script, page_size=5000)
            data = pd.DataFrame(res['data'])
            index_list.append(data)
        index_df = pd.concat(index_list).rename(columns={"TRADEDATE": "trade_date"})
        index_df['trade_date'] = index_df['trade_date'].astype(str)
        index_df['INDEXNAME'] = index_df['TICKER'].map(map_dict)
        index_df = pd.pivot_table(index_df, index="trade_date", columns="INDEXNAME", values="TCLOSE").sort_index()
        trading_day_list = get_trading_day_list(self.start_date, self.end_date)
        assert (index_df.shape[0] == len(trading_day_list))

        index_df = index_df.reindex(trading_day_list)

        self.index_df = index_df

    def _load_market_status_ratio(self):
        date_list = get_trading_day_list(self.start_date, self.end_date, frequency="month")
        market_status = pd.DataFrame(index=date_list, columns=['估值水平'])
        # 估值水平
        data = pd.read_csv(
            "D:\\研究基地\\G-专题报告\\【2023.3】四象限风格轮动\\data\\hs300_pe_and_pb.csv", dtype={"trade_date": str})
        pe_data = data.set_index('trade_date')
        valuation = pe_data.rolling(
            window=252*5, min_periods=252*5).apply(lambda x: x.rank(pct=True).iloc[-1]).reindex(date_list)
        market_status.loc[:, '估值水平'] = valuation.mean(axis=1)
        # 风险溢价
        sql_script = "SELECT * FROM mac_treasury_yield WHERE TRADE_DATE <= {}".format(self.end_date)
        engine = create_engine(engine_params)
        data = pd.read_sql(sql_script, engine)
        data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        data = data.set_index('trade_date')[['ytm_10y']].dropna().sort_index()
        data = data.merge(pe_data, left_index=True, right_index=True)
        data['risk_premium'] = (1. / data['PE_TTM']) - (data['ytm_10y'] / 100.)
        risk_premium = data['risk_premium'].rolling(
            window=252*5, min_periods=252*5).apply(lambda x: x.rank(pct=True).iloc[-1]).reindex(date_list)
        market_status.loc[:, '风险溢价'] = risk_premium
        # 指数收益率 & 指数波动率
        index_df = self.index_df.copy()
        index_return = index_df.pct_change().dropna()
        index_return['trade_date'] = index_return.index
        index_return['year-month'] = index_return['trade_date'].apply(lambda x: str(
            datetime.strptime(x, "%Y%m%d").year) + '.' + str(datetime.strptime(x, "%Y%m%d").month).zfill(2))
        index_return['成长'] = 0.5 * (index_return['大盘成长'] + index_return['小盘成长'])
        index_return['价值'] = 0.5 * (index_return['大盘价值'] + index_return['小盘价值'])
        include_list = ['大盘成长', '大盘价值', '小盘成长', '小盘价值', '沪深300', '中证500', '中证1000', '成长', '价值']
        m_ret = index_return.groupby('year-month')[include_list].apply(lambda x: (1 + x).prod() - 1).sort_index()
        m_ret.eval("ret_spread = 沪深300 - 中证1000", inplace=True)
        m_ret.eval("spread = 沪深300 - 中证500", inplace=True)
        m_ret.eval("Y = 成长 - 价值", inplace=True)
        m_ret.eval("Y1 = 大盘成长 - 大盘价值", inplace=True)
        m_ret.eval("Y2 = 大盘成长 - 小盘价值", inplace=True)
        m_ret.eval("Y3 = 大盘成长 - 小盘成长", inplace=True)
        m_ret.eval("Y4 = 大盘价值 - 小盘价值", inplace=True)
        m_ret.eval("Y5 = 大盘价值 - 小盘成长", inplace=True)
        m_ret.eval("Y6 = 小盘成长 - 小盘价值", inplace=True)
        m_vol = index_return.groupby('year-month')[include_list].apply(lambda x: x.std() * np.sqrt(252))
        m_vol.eval("vol_spread = 沪深300 - 中证1000", inplace=True)
        m_vol.eval("spread = 沪深300 - 中证500", inplace=True)
        m_vol.eval("Y = 成长 - 价值", inplace=True)
        market_status.loc[:, '300收益率'] = m_ret['沪深300'].tolist()
        market_status.loc[:, '500收益率'] = m_ret['中证500'].tolist()
        market_status.loc[:, '1000收益率'] = m_ret['中证1000'].tolist()
        market_status.loc[:, "大小盘收益差值"] = m_ret['ret_spread'].tolist()
        market_status.loc[:, '300收益波动率'] = m_vol['沪深300'].tolist()
        market_status.loc[:, '1000收益波动率'] = m_vol['中证1000'].tolist()
        market_status.loc[:, '大小盘波动差值'] = m_vol['vol_spread'].tolist()
        market_status.loc[:, "300与500收益率差值"] = m_ret['spread'].tolist()
        market_status.loc[:, "300与500波动率差值"] = m_vol['spread'].tolist()
        market_status.loc[:, "成长价值收益差值"] = m_ret["Y"].tolist()
        market_status.loc[:, "成长价值波动差值"] = m_vol["Y"].tolist()
        market_status.loc[:, "大盘成长-大盘价值"] = m_ret['Y1'].tolist()
        market_status.loc[:, "大盘成长-小盘价值"] = m_ret['Y2'].tolist()
        market_status.loc[:, "大盘成长-小盘成长"] = m_ret['Y3'].tolist()
        market_status.loc[:, "大盘价值-大盘价值"] = m_ret['Y4'].tolist()
        market_status.loc[:, "大盘价值-小盘成长"] = m_ret['Y5'].tolist()
        market_status.loc[:, "小盘成长-小盘价值"] = m_ret['Y6'].tolist()
        # 成交额&换手率
        sql_script = "SELECT * FROM mac_stock_trading"
        engine = create_engine(engine_params)
        data = pd.read_sql(sql_script, engine)
        data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
        data = data[['trade_date', 'turn_300', 'turn_1000', 'amt_sh', 'amt_sz']]
        data.eval("amt_all = amt_sh + amt_sz", inplace=True)
        data['turn_ratio'] = data['turn_300'] / data['turn_1000']
        data['大小盘相对换手率'] = data['turn_ratio'].rolling(60).mean()
        data = data.set_index('trade_date')
        market_status.loc[:, "大小盘相对换手率"] = data.reindex(date_list)['大小盘相对换手率']
        market_status.loc[:, "沪深成交额分位"] = data['amt_all'].rolling(
            window=252*2, min_periods=252*2).apply(lambda x: x.rank(pct=True).iloc[-1]).reindex(date_list)
        # Wind全A换手率
        data = pd.read_csv(
            "D:\\研究基地\\G-专题报告\\【2023.3】四象限风格轮动\\data\\Wind_A_turn.csv", dtype={"trade_date": str})
        data = data.set_index('trade_date')
        market_status.loc[:, "Wind全A换手分位"] = data['TURN'].rolling(
            window=252*2, min_periods=252*2).apply(lambda x: x.rank(pct=True).iloc[-1]).reindex(date_list)
        # AR + pb
        index_codes = ['399372', '399373', '399376', '399377', '000300', '000852']
        ar_list = []
        pb_list = []
        for index_code in index_codes:
            sql_script = "SELECT jyrq as TRADEDATE, zqdm as TICKER, kpjg as TOPEN, zgjg as HIGH, " \
                         "zdjg as LOW, spjg as TCLOSE, pb FROM st_market.t_st_zs_hqql WHERE " \
                         "ZQDM = '{}' and JYRQ >= {} and " \
                         "JYRQ <= {}".format(index_code, self.start_date, self.end_date)
            res = hbs.db_data_query("alluser", sql_script, page_size=5000)
            data = pd.DataFrame(res['data']).rename(columns={"TRADEDATE": "trade_date"})
            data['trade_date'] = data['trade_date'].astype(str)
            data = data.set_index('trade_date')[['TOPEN', 'HIGH', 'LOW', 'pb']]
            pb_list.append(data['pb'].to_frame(map_dict[index_code]))
            data.eval("x1 = HIGH - TOPEN", inplace=True)
            data.eval("x2 = TOPEN - LOW", inplace=True)
            ar_ratio = \
                data.rolling(250)['x1'].sum() / data.rolling(250)['x2'].sum()
            ar_ratio = ar_ratio.reindex(date_list).to_frame(map_dict[index_code])
            ar_list.append(ar_ratio)
        ar_df = pd.concat(ar_list, axis=1)
        market_status = market_status.merge(ar_df, left_index=True, right_index=True)
        pb_df = pd.concat(pb_list, axis=1)
        pb_df.loc[:"20141016", '中证1000'] = np.NaN
        pb_df['大盘PB/小盘PB'] = pb_df['沪深300'] / pb_df['中证1000']
        pb_df['大盘成长PB/大盘价值PB'] = pb_df['大盘成长'] / pb_df['大盘价值']
        pb_df['小盘成长PB/小盘价值PB'] = pb_df['小盘成长'] / pb_df['小盘价值']
        pb_df = pb_df[pb_df.columns[-3:]]
        pb_quantile1 = pb_df[['大盘PB/小盘PB']].dropna().rolling(
            window=252*2, min_periods=252*2).apply(lambda x: x.rank(pct=True).iloc[-1]).reindex(date_list)
        pb_quantile2 = pb_df[['大盘成长PB/大盘价值PB', '小盘成长PB/小盘价值PB']].dropna().rolling(
            window=252*2, min_periods=252*2).apply(lambda x: x.rank(pct=True).iloc[-1]).reindex(date_list)
        pb_quantile = pb_quantile1.merge(pb_quantile2, left_index=True, right_index=True)
        pb_df = pb_df.merge(pb_quantile, left_index=True, right_index=True, suffixes=('_init', '_quantile'))
        market_status = market_status.merge(pb_df, left_index=True, right_index=True)

        self.market_status = market_status

    def _load_market_sentiments(self):
        pass

    def _load_macro_env(self):
        pass

    def _load_data(self):
        self._load_style_index()
        self._load_market_status_ratio()
        self._load_market_sentiments()
        self._load_macro_env()

    @staticmethod
    def corr_test(x, y, shift_window):
        corr_list = []
        for col in x.columns:
            df = x[[col]].merge(y.shift(-shift_window), left_index=True, right_index=True).dropna()
            tmp = df.corr('pearson')[col][1:].to_frame(col)
            corr_list.append(tmp)
        corr_df = pd.concat(corr_list, axis=1)

        return corr_df

    @staticmethod
    def granger_test(x, y):
        granger_1 = pd.DataFrame(index=x.columns, columns=y.columns)
        granger_2 = pd.DataFrame(index=x.columns, columns=y.columns)
        for col in x.columns:
            df = x[[col]].merge(y, left_index=True, right_index=True).dropna()
            for col2 in y.columns:
                result = sm.tsa.stattools.grangercausalitytests(df[[col2, col]], maxlag=2, verbose=False)
                granger_1.loc[col, col2] = result[1][0]['ssr_ftest'][1]
                granger_2.loc[col, col2] = result[2][0]['ssr_ftest'][1]

        return granger_1, granger_2

    def run_analysis(self):
        market_status = self.market_status.copy()
        # 相关性检验
        predict_df = market_status[["大小盘收益差值", "成长价值收益差值", "大盘价值-小盘成长", "大盘成长-小盘价值"]]
        predict_df.rename(columns={"大小盘收益差值": "Y1", "成长价值收益差值": "Y2",
                                   "大盘价值-小盘成长": "Y3", "大盘成长-小盘价值": "Y4"}, inplace=True)
        corr_status_shift_1 = self.corr_test(market_status, predict_df, shift_window=1)
        corr_status_shift_2 = self.corr_test(market_status, predict_df, shift_window=2)
        corr_status_shift_3 = self.corr_test(market_status, predict_df, shift_window=3)

        granger_1, granger_2 = self.granger_test(market_status, predict_df)

        return corr_status_shift_1, corr_status_shift_2, corr_status_shift_3, granger_1, granger_2


if __name__ == '__main__':
    StyleRotation('20100101', '20230228').run_analysis()



"""
九坤多空敞口测算
"""
import pandas as pd
import hbshare as hbs
from hbshare.quant.Kevin.quant_room.MyUtil.data_loader import get_fund_nav_from_sql, get_trading_day_list


def hedge_ratio_calc(start_date, end_date):
    nav_df = get_fund_nav_from_sql(start_date, end_date, {"日享": "ST9804", "多空": "SEF596", "中性": "S84924"})
    trading_day_list = get_trading_day_list(start_date, end_date, frequency="week")
    nav_df = nav_df.reindex(trading_day_list).dropna()
    return_df = nav_df.pct_change().dropna()
    # 指数收益
    sql_script = "SELECT JYRQ as TRADEDATE, ZQMC as INDEXNAME, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM = '{}' " \
                 "and JYRQ >= {} and JYRQ <= {}".format('000905', start_date, end_date)
    data = pd.DataFrame(hbs.db_data_query('readonly', sql_script)['data'])
    index_return = data.set_index('TRADEDATE').reindex(trading_day_list)['TCLOSE'].pct_change().dropna()
    # 整合
    return_df = return_df.merge(index_return.to_frame('beta'), left_index=True, right_index=True)
    return_df['对冲成本'] = return_df['日享'] - return_df['beta'] - return_df['中性'] * 1.2
    return_df['hedge_ratio'] = (return_df['日享'] - return_df['多空']) / (return_df['beta'] + return_df['对冲成本'])
    return_df['tmp'] = (1 + return_df['beta']).cumprod()
    return_df['period'] = return_df['tmp'].pct_change(periods=4)

    # df = return_df[(return_df['hedge_ratio'] >= 0.2) & (return_df['hedge_ratio'] <= 1)]
    # df['mean_ratio'] = df['hedge_ratio'].rolling(4).mean()
    # df = df.dropna()

    # print(return_df[(return_df['hedge_ratio'] >= 0.2) & (return_df['hedge_ratio'] <= 1)].shape[0])

    return return_df


if __name__ == '__main__':
    hedge_ratio_calc("20181228", "20230120")


# -*- coding: utf-8 -*-

from hbshare.fe.xwq.analysis.orm.fedb import FEDB
from hbshare.fe.xwq.analysis.orm.hbdb import HBDB
from datetime import datetime, timedelta
import numpy as np
from matplotlib.ticker import FuncFormatter
import matplotlib.pyplot as plt
import seaborn as sns
plt.rcParams['font.sans-serif'] = ['kaiti']  # 用来正常显示中文标签
plt.rcParams['axes.unicode_minus'] = False  # 用来正常显示负号
line_color_list = ['#F04950', '#6268A2', '#959595', '#333335', '#EE703F', '#7E4A9B', '#8A662C',
                  '#44488E', '#BA67E9', '#3FAEEE']
bar_color_list = ['#C94649', '#EEB2B4', '#E1777A', '#D57C56', '#E39A79', '#DB8A66', '#E5B88C',
                  '#8588B7', '#B4B6D1', '#55598D', '#628497', '#A9C6CB', '#866EA9', '#B79BC7',
                  '#7D7D7E', '#CACACA', '#A7A7A8', '#606063', '#C4C4C4', '#99999B', '#B7B7B7']
area_color_list = ['#D55659', '#E1777A', '#DB8A66', '#E5B88C', '#EEB2B4', '#D57C56', '#E39A79',
                   '#8588B7', '#626697', '#866EA9', '#B79BC7', '#B4B6D1', '#628497', '#A9C6CB',
                   '#7D7D7E', '#A7A7A8', '#99999B', '#B7B7B7', '#CACACA', '#969696', '#C4C4C4']
new_color_list = ['#F04950', '#959595', '#6268A2', '#333335', '#D57C56', '#628497']


def to_percent(temp, position):
    return '%1.0f'%(temp) + '%'

def to_percent_r2(temp, position):
    return '%0.01f'%(temp) + '%'

def to_100percent(temp, position):
    return '%1.0f'%(temp * 100) + '%'

def to_100percent_r2(temp, position):
    return '%0.01f'%(temp * 100) + '%'

def get_date(start_date, end_date):
    calendar_df = HBDB().read_cal(start_date, end_date)
    calendar_df = calendar_df.rename(columns={'JYRQ': 'CALENDAR_DATE', 'SFJJ': 'IS_OPEN', 'SFZM': 'IS_WEEK_END', 'SFYM': 'IS_MONTH_END'})
    calendar_df['CALENDAR_DATE'] = calendar_df['CALENDAR_DATE'].astype(str)
    calendar_df = calendar_df.sort_values('CALENDAR_DATE')
    calendar_df['IS_OPEN'] = calendar_df['IS_OPEN'].astype(int).replace({0: 1, 1: 0})
    calendar_df['YEAR_MONTH'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[:6])
    calendar_df['MONTH'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[4:6])
    calendar_df['MONTH_DAY'] = calendar_df['CALENDAR_DATE'].apply(lambda x: x[4:])
    calendar_df = calendar_df[(calendar_df['CALENDAR_DATE'] >= start_date) & (calendar_df['CALENDAR_DATE'] <= end_date)]
    trade_df = calendar_df[calendar_df['IS_OPEN'] == 1].rename(columns={'CALENDAR_DATE': 'TRADE_DATE'})
    trade_df = trade_df[(trade_df['TRADE_DATE'] >= start_date) & (trade_df['TRADE_DATE'] <= end_date)]
    report_df = calendar_df.drop_duplicates('YEAR_MONTH', keep='last').rename(columns={'CALENDAR_DATE': 'REPORT_DATE'})
    report_df = report_df[report_df['MONTH_DAY'].isin(['0331', '0630', '0930', '1231'])]
    report_df = report_df[(report_df['REPORT_DATE'] >= start_date) & (report_df['REPORT_DATE'] <= end_date)]
    report_trade_df = calendar_df[calendar_df['IS_OPEN'] == 1].rename(columns={'CALENDAR_DATE': 'TRADE_DATE'})
    report_trade_df = report_trade_df.sort_values('TRADE_DATE').drop_duplicates('YEAR_MONTH', keep='last')
    report_trade_df = report_trade_df[report_trade_df['MONTH'].isin(['03', '06', '09', '12'])]
    report_trade_df = report_trade_df[(report_trade_df['TRADE_DATE'] >= start_date) & (report_trade_df['TRADE_DATE'] <= end_date)]
    calendar_trade_df = calendar_df[['CALENDAR_DATE']].merge(trade_df[['TRADE_DATE']], left_on=['CALENDAR_DATE'], right_on=['TRADE_DATE'], how='left')
    calendar_trade_df['TRADE_DATE'] = calendar_trade_df['TRADE_DATE'].fillna(method='ffill')
    calendar_trade_df = calendar_trade_df[(calendar_trade_df['TRADE_DATE'] >= start_date) & (calendar_trade_df['TRADE_DATE'] <= end_date)]
    return calendar_df, report_df, trade_df, report_trade_df, calendar_trade_df

def quantile_definition(idxs, col, daily_df):
    part_df = daily_df.iloc[list(map(int, idxs))].copy(deep=True)
    q = (1.0 - np.count_nonzero(part_df[col].iloc[-1] <= part_df[col]) / len(part_df))
    return q


class TimingAnalysis:
    def __init__(self, start_date, end_date, data_path):
        self.start_date = start_date
        self.end_date = end_date
        self.data_path = data_path
        self.calendar_df, self.report_df, self.trade_df, self.report_trade_df, self.calendar_trade_df = get_date('19000101', self.end_date)

    def MarketTiming(self):
        # 成交额占比
        turnover_proportion = FEDB().read_timing_data(['TRADE_DATE', 'TURNOVER_PROPORTION_000300', 'TURNOVER_PROPORTION_000905', 'TURNOVER_PROPORTION_000852'], 'timing_market', self.start_date, self.end_date)
        turnover_proportion['TRADE_DATE'] = turnover_proportion['TRADE_DATE'].astype(str)
        turnover_proportion = turnover_proportion[turnover_proportion['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        turnover_proportion = turnover_proportion.sort_values('TRADE_DATE')
        turnover_proportion['TRADE_DATE'] = turnover_proportion['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(turnover_proportion['TRADE_DATE'].values, turnover_proportion['TURNOVER_PROPORTION_000300'].values, color=line_color_list[0], label='沪深300')
        plt.plot(turnover_proportion['TRADE_DATE'].values, turnover_proportion['TURNOVER_PROPORTION_000905'].values, color=line_color_list[1], label='中证500')
        plt.plot(turnover_proportion['TRADE_DATE'].values, turnover_proportion['TURNOVER_PROPORTION_000852'].values, color=line_color_list[2], label='中证1000')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=3)
        plt.gca().yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.title('成交额占比', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '成交额占比'))

        # 北向资金净流入
        hk_cash = FEDB().read_timing_data(['TRADE_DATE', 'SH_NET_PURCHASE', 'SZ_NET_PURCHASE', 'HK_NET_PURCHASE'], 'timing_market', '20071231', self.end_date)
        hk_cash['TRADE_DATE'] = hk_cash['TRADE_DATE'].astype(str)
        hk_cash = hk_cash.set_index('TRADE_DATE').dropna(how='all').reset_index()
        hk_cash = hk_cash.sort_values('TRADE_DATE')
        hk_cash['SH_NET_PURCHASE'] = hk_cash['SH_NET_PURCHASE'].cumsum()
        hk_cash['SZ_NET_PURCHASE'] = hk_cash['SZ_NET_PURCHASE'].cumsum()
        hk_cash['HK_NET_PURCHASE'] = hk_cash['HK_NET_PURCHASE'].cumsum()
        hk_cash['TRADE_DATE'] = hk_cash['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(hk_cash['TRADE_DATE'].values, hk_cash['HK_NET_PURCHASE'].values, color=line_color_list[0], label='北向资金净流入（亿元）')
        plt.plot(hk_cash['TRADE_DATE'].values, hk_cash['SH_NET_PURCHASE'].values, color=line_color_list[1], label='沪股通净流入（亿元）')
        plt.plot(hk_cash['TRADE_DATE'].values, hk_cash['SZ_NET_PURCHASE'].values, color=line_color_list[2], label='深股通净流入（亿元）')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=3)
        plt.title('北向资金累计净流入', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '北向资金累计净流入'))

        # 融资融券余额
        margin_balance = FEDB().read_timing_data(['TRADE_DATE', 'MARGIN_BALANCE'], 'timing_market', '20071231', self.end_date)
        margin_balance['TRADE_DATE'] = margin_balance['TRADE_DATE'].astype(str)
        margin_balance = margin_balance.set_index('TRADE_DATE').dropna(how='all').reset_index()
        margin_balance = margin_balance.sort_values('TRADE_DATE')
        margin_balance['TRADE_DATE'] = margin_balance['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.stackplot(margin_balance['TRADE_DATE'].values, margin_balance['MARGIN_BALANCE'].values, color=line_color_list[0], alpha=0.5, labels=['融资融券余额（亿元）'])
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=1)
        plt.title('融资融券余额', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '融资融券余额'))

        # 创近一月/近六月新高个股数量及占比
        new_high = FEDB().read_timing_data(['TRADE_DATE', 'NEW_HIGH_SHORT', 'NEW_HIGH_RATIO_SHORT', 'NEW_HIGH_LONG', 'NEW_HIGH_RATIO_LONG'], 'timing_market', self.start_date, self.end_date)
        new_high['TRADE_DATE'] = new_high['TRADE_DATE'].astype(str)
        new_high = new_high[new_high['TRADE_DATE'].isin(self.trade_df[self.trade_df['IS_MONTH_END'] == '1']['TRADE_DATE'].unique().tolist())]
        new_high = new_high.sort_values('TRADE_DATE')
        fig, ax1 = plt.subplots(figsize=(12, 6))
        ax2 = ax1.twinx()
        bar_width = 0.3
        ax1.bar(np.arange(len(new_high)) - 0.5 * bar_width, new_high['NEW_HIGH_SHORT'].values, bar_width, label='创近一月新高个股数量', color=bar_color_list[0])
        ax1.bar(np.arange(len(new_high)) + 0.5 * bar_width, new_high['NEW_HIGH_LONG'].values, bar_width, label='创近六月新高个股数量', color=bar_color_list[7])
        ax2.plot(np.arange(len(new_high)), new_high['NEW_HIGH_RATIO_SHORT'].values, label='创近一月新高个股占比（右轴）', color=line_color_list[0])
        ax2.plot(np.arange(len(new_high)), new_high['NEW_HIGH_RATIO_LONG'].values, label='创近六月新高个股占比（右轴）',  color=line_color_list[1])
        h1, l1 = ax1.get_legend_handles_labels()
        h2, l2 = ax2.get_legend_handles_labels()
        plt.legend(handles=h1 + h2, labels=l1 + l2, loc=8, bbox_to_anchor=(0.5, -0.25), ncol=4)
        ax1.set_xticks(np.arange(len(new_high)))
        ax1.set_xticklabels(labels=new_high['TRADE_DATE'].unique().tolist(), rotation=60)
        ax2.yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.title('创新高个股数量及占比', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=False, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '创新高个股数量及占比'))

        # 行业分歧度
        industry_divergence = FEDB().read_timing_data(['TRADE_DATE', 'TURNOVER_VALUE', 'PCA_1ST_SHORT', 'PCA_1ST_LONG'], 'timing_market', self.start_date, self.end_date)
        industry_divergence['TRADE_DATE'] = industry_divergence['TRADE_DATE'].astype(str)
        industry_divergence = industry_divergence[industry_divergence['TRADE_DATE'].isin(self.trade_df[self.trade_df['IS_WEEK_END'] == '1']['TRADE_DATE'].unique().tolist())]
        industry_divergence = industry_divergence.sort_values('TRADE_DATE')
        industry_divergence['TURNOVER_VALUE'] = industry_divergence['TURNOVER_VALUE'] / 100000000.0
        industry_divergence['TRADE_DATE'] = industry_divergence['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax1 = plt.subplots(figsize=(12, 6))
        ax2 = ax1.twinx()
        ax1.plot(industry_divergence['TRADE_DATE'].values, industry_divergence['PCA_1ST_SHORT'].values, color=line_color_list[0], label='近10日行业分歧度')
        ax1.plot(industry_divergence['TRADE_DATE'].values, industry_divergence['PCA_1ST_LONG'].values, color=line_color_list[1], label='近60日行业分歧度')
        ax2.stackplot(industry_divergence['TRADE_DATE'].values, industry_divergence['TURNOVER_VALUE'].values, color=line_color_list[3], alpha=0.5, labels=['市场成交额（亿元， 右轴）'])
        h1, l1 = ax1.get_legend_handles_labels()
        h2, l2 = ax2.get_legend_handles_labels()
        plt.legend(handles=h1 + h2, labels=l1 + l2, loc=8, bbox_to_anchor=(0.5, -0.15), ncol=3)
        ax1.set_ylim([0, 1])
        ax2.set_ylim([0, 30000])
        ax1.yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.title('行业分歧度', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=False, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '行业分歧度'))

        # 偏股基金仓位（公募、私募）
        mutual_position = FEDB().read_timing_data(['TRADE_DATE', 'POSITION_MUTUAL'], 'timing_market', self.start_date, self.end_date)
        mutual_position['TRADE_DATE'] = mutual_position['TRADE_DATE'].astype(str)
        mutual_position = mutual_position[mutual_position['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        mutual_position = mutual_position.sort_values('TRADE_DATE')
        mutual_position['TRADE_DATE'] = mutual_position['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        private_position = FEDB().read_timing_data(['TRADE_DATE', 'POSITION_PRIVATE'], 'timing_market', self.start_date, self.end_date)
        private_position['TRADE_DATE'] = private_position['TRADE_DATE'].astype(str)
        private_position = private_position[private_position['TRADE_DATE'].isin(self.trade_df[self.trade_df['IS_WEEK_END'] == '1']['TRADE_DATE'].unique().tolist())]
        private_position = private_position.sort_values('TRADE_DATE')
        private_position['TRADE_DATE'] = private_position['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(mutual_position['TRADE_DATE'].values, mutual_position['POSITION_MUTUAL'].values, color=line_color_list[0], label='偏股混合型公募基金仓位')
        plt.plot(private_position['TRADE_DATE'].values, private_position['POSITION_PRIVATE'].values, color=line_color_list[1], label='主观多头型私募基金仓位')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=2)
        plt.gca().yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.title('偏股基金仓位', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '偏股基金仓位'))

        # 市场景气度
        prosperity = FEDB().read_timing_data(['TRADE_DATE', 'NET_PROFIT_YOY_000300', 'NET_PROFIT_YOY_000905', 'NET_PROFIT_YOY_000852', 'NET_PROFIT_YOY_000985'], 'timing_market', self.start_date, self.end_date)
        prosperity['TRADE_DATE'] = prosperity['TRADE_DATE'].astype(str)
        prosperity = prosperity[prosperity['TRADE_DATE'].isin(self.report_df['REPORT_DATE'].unique().tolist())]
        prosperity = prosperity.sort_values('TRADE_DATE')
        fig, ax = plt.subplots(figsize=(12, 6))
        bar_width = 0.2
        ax.bar(np.arange(len(prosperity)) - 1.5 * bar_width, prosperity['NET_PROFIT_YOY_000300'].values, bar_width, label='沪深300', color=new_color_list[0])
        ax.bar(np.arange(len(prosperity)) - 0.5 * bar_width, prosperity['NET_PROFIT_YOY_000905'].values, bar_width, label='中证500', color=new_color_list[2])
        ax.bar(np.arange(len(prosperity)) + 0.5 * bar_width, prosperity['NET_PROFIT_YOY_000852'].values, bar_width, label='中证1000', color=new_color_list[1])
        ax.bar(np.arange(len(prosperity)) + 1.5 * bar_width, prosperity['NET_PROFIT_YOY_000985'].values, bar_width, label='中证全指', color=new_color_list[3])
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.25), ncol=4)
        ax.set_xticks(np.arange(len(prosperity)))
        ax.set_xticklabels(labels=prosperity['TRADE_DATE'].unique().tolist(), rotation=60)
        ax.yaxis.set_major_formatter(FuncFormatter(to_percent))
        plt.title('净利润同比增速', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '净利润同比增速'))

        # 股债性价比
        bond_stock = FEDB().read_timing_data(['TRADE_DATE', 'B_GXL_000300', 'B_GXL_000905', 'B_GXL_000852', 'B_GXL_000985'], 'timing_market', self.start_date, self.end_date)
        bond_stock['TRADE_DATE'] = bond_stock['TRADE_DATE'].astype(str)
        bond_stock = bond_stock[bond_stock['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        bond_stock = bond_stock.sort_values('TRADE_DATE')
        bond_stock['TRADE_DATE'] = bond_stock['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(bond_stock['TRADE_DATE'].values, bond_stock['B_GXL_000300'].values, color=line_color_list[0], label='沪深300')
        plt.plot(bond_stock['TRADE_DATE'].values, bond_stock['B_GXL_000905'].values, color=line_color_list[1], label='中证500')
        plt.plot(bond_stock['TRADE_DATE'].values, bond_stock['B_GXL_000852'].values, color=line_color_list[2], label='中证1000')
        plt.plot(bond_stock['TRADE_DATE'].values, bond_stock['B_GXL_000985'].values, color=line_color_list[3], label='中证全指')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=4)
        plt.gca().yaxis.set_major_formatter(FuncFormatter(to_percent_r2))
        plt.title('股债性价比', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '股债性价比'))

        # market_timing = FEDB().read_timing_data(['TRADE_DATE', 'TURNOVER_VALUE', 'TURNOVER_PROPORTION_000300', 'TURNOVER_PROPORTION_000905', 'TURNOVER_PROPORTION_000852', 'TURNOVER_PROPORTION_000985',
        #                                          'SH_NET_PURCHASE', 'SZ_NET_PURCHASE', 'HK_NET_PURCHASE', 'MARGIN_BALANCE', 'NEW_HIGH_SHORT', 'NEW_HIGH_RATIO_SHORT', 'NEW_HIGH_LONG', 'NEW_HIGH_RATIO_LONG',
        #                                          'PCA_1ST_SHORT', 'PCA_1ST_LONG', 'POSITION_MUTUAL', 'POSITION_PRIVATE',
        #                                          'NET_PROFIT_YOY_000300', 'NET_PROFIT_YOY_000905', 'NET_PROFIT_YOY_000852', 'NET_PROFIT_YOY_000985',
        #                                          'EP_B_000300', 'EP_B_000905', 'EP_B_000852', 'EP_B_000985',
        #                                          'B_GXL_000300', 'B_GXL_000905', 'B_GXL_000852', 'B_GXL_000985'],
        #                                          'timing_market', (datetime.strptime(self.start_date, '%Y%m%d') - timedelta(365 * 2)).strftime('%Y%m%d'), self.end_date)
        # market_timing['TRADE_DATE'] = market_timing['TRADE_DATE'].astype(str)
        # market_timing = market_timing.sort_values('TRADE_DATE')
        # market_timing = market_timing.fillna(method='ffill')
        # market_timing = market_timing.set_index('TRADE_DATE')
        #
        # market_crowding = market_timing[['TURNOVER_PROPORTION_000300', 'TURNOVER_PROPORTION_000905', 'TURNOVER_PROPORTION_000852', 'TURNOVER_PROPORTION_000985',
        #                                  'SH_NET_PURCHASE', 'SZ_NET_PURCHASE', 'HK_NET_PURCHASE', 'MARGIN_BALANCE', 'NEW_HIGH_RATIO_SHORT', 'NEW_HIGH_RATIO_LONG',
        #                                  'PCA_1ST_SHORT', 'PCA_1ST_LONG', 'POSITION_MUTUAL', 'POSITION_PRIVATE']]
        # market_crowding = market_crowding[market_crowding.index.isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        # market_crowding['MARGIN_BALANCE'] = market_crowding['MARGIN_BALANCE'].diff()
        # market_crowding['IDX'] = range(len(market_crowding))
        # for col in list(market_crowding.columns):
        #     market_crowding[col] = market_crowding['IDX'].rolling(window=250, min_periods=250, center=False).apply(lambda x: quantile_definition(x, col, market_crowding))
        # market_crowding['CROWDING'] = ((market_crowding['TURNOVER_PROPORTION_000300'] + market_crowding['TURNOVER_PROPORTION_000905'] + market_crowding['TURNOVER_PROPORTION_000852'] + market_crowding['TURNOVER_PROPORTION_000985']) * (1.0 / 4.0) \
        #                             + (market_crowding['SH_NET_PURCHASE'] + market_crowding['SZ_NET_PURCHASE'] + market_crowding['HK_NET_PURCHASE']) * (1.0 / 3.0) \
        #                             + market_crowding['MARGIN_BALANCE'] \
        #                             + (market_crowding['NEW_HIGH_RATIO_SHORT'] + market_crowding['NEW_HIGH_RATIO_LONG']) * (1.0 / 2.0) \
        #                             + (market_crowding['PCA_1ST_SHORT'] + market_crowding['PCA_1ST_LONG']) * (1.0 / 2.0) \
        #                             + (market_crowding['POSITION_MUTUAL'] + market_crowding['POSITION_PRIVATE']) * (1.0 / 2.0)) * (1.0 / 6.0)
        # market_crowding = market_crowding.dropna(subset=['CROWDING'])
        # market_crowding['CROWDING_'] = market_crowding['CROWDING'].rolling(window=10, min_periods=1, center=False).mean()
        # market_crowding['CROWDING_MEAN'] = market_crowding['CROWDING'].rolling(window=250, min_periods=1, center=False).mean()
        # market_crowding['CROWDING_UP1'] = market_crowding['CROWDING'].rolling(window=250, min_periods=1,  center=False).mean() + 1.0 * market_crowding['CROWDING'].rolling(window=250, min_periods=1, center=False).std(ddof=1)
        # market_crowding['CROWDING_DOWN1'] = market_crowding['CROWDING'].rolling(window=250, min_periods=1, center=False).mean() - 1.0 * market_crowding['CROWDING'].rolling(window=250, min_periods=1, center=False).std(ddof=1)
        # market_crowding['CROWDING_UP15'] = market_crowding['CROWDING'].rolling(window=250, min_periods=1, center=False).mean() + 1.5 * market_crowding['CROWDING'].rolling(window=250, min_periods=1, center=False).std(ddof=1)
        # market_crowding['CROWDING_DOWN15'] = market_crowding['CROWDING'].rolling(window=250, min_periods=1, center=False).mean() - 1.5 * market_crowding['CROWDING'].rolling(window=250, min_periods=1, center=False).std(ddof=1)
        # index_daily_k = HBDB().read_index_daily_k_given_date_and_indexs((datetime.strptime(self.start_date, '%Y%m%d') - timedelta(365 * 2)).strftime('%Y%m%d'), ['881001'])
        # index_daily_k = index_daily_k[['jyrq', 'spjg']]
        # index_daily_k.columns = ['TRADE_DATE', 'CLOSE_INDEX']
        # index_daily_k['TRADE_DATE'] = index_daily_k['TRADE_DATE'].astype(str)
        # index_daily_k = index_daily_k.sort_values('TRADE_DATE')
        # index_daily_k = index_daily_k.set_index('TRADE_DATE')
        # market_crowding = market_crowding.merge(index_daily_k, left_index=True, right_index=True, how='left')
        # market_crowding = market_crowding[(market_crowding.index >= self.start_date) & (market_crowding.index <= self.end_date)]
        # market_crowding.index = map(lambda x: datetime.strptime(x, '%Y%m%d'), market_crowding.index)
        #
        # market_crowding.loc[market_crowding['CROWDING_'] <= market_crowding['CROWDING_DOWN15'], 'BUY_SELL'] = 1
        # market_crowding.loc[market_crowding['CROWDING_'] >= market_crowding['CROWDING_UP15'], 'BUY_SELL'] = 0\
        #
        # market_crowding['BUY_SELL'] = market_crowding['BUY_SELL'].fillna(method='ffill')
        # market_crowding['BUY_SELL'] = market_crowding['BUY_SELL'].fillna(1)
        # market_crowding['BUY_SELL_CHANGE'] = market_crowding['BUY_SELL'].diff()
        # market_crowding['BUY_SELL_CHANGE'] = market_crowding['BUY_SELL_CHANGE'].fillna(1)
        # market_crowding_buy = market_crowding[market_crowding['BUY_SELL_CHANGE'] == 1]
        # market_crowding['SELL'] = market_crowding['CLOSE_INDEX'] * market_crowding['BUY_SELL_CHANGE'] * (-1)
        #
        # fig, ax = plt.subplots(figsize=(12, 6))
        # ax1 = ax
        # ax2 = ax.twinx()
        # ax1.plot(market_crowding.index, market_crowding['CROWDING_'].values, '-', linewidth=3.0, label='拥挤度指标', color=line_color_list[1])
        # ax1.plot(market_crowding.index, market_crowding['CROWDING_MEAN'].values, '--', label='均值', color=line_color_list[2])
        # ax1.plot(market_crowding.index, market_crowding['CROWDING_UP1'].values, '--', label='均值±1.0*标准差', color=line_color_list[7])
        # ax1.plot(market_crowding.index, market_crowding['CROWDING_UP15'].values, '--', label='均值±1.5*标准差', color=line_color_list[3])
        # ax1.plot(market_crowding.index, market_crowding['CROWDING_DOWN1'].values, '--', color=line_color_list[7])
        # ax1.plot(market_crowding.index, market_crowding['CROWDING_DOWN15'].values, '--', color=line_color_list[3])
        # ax2.plot(market_crowding.index, market_crowding['CLOSE_INDEX'].values, '-', linewidth=3.0, label='万得全A（右轴）', color=line_color_list[5])
        # ax2.plot(market_crowding.index, market_crowding['CLOSE_INDEX'].values, '-', label='买入点', color='r')
        # ax2.plot(market_crowding.index, market_crowding['CLOSE_INDEX'].values, '-', label='卖出点', color='g')
        # h1, l1 = ax1.get_legend_handles_labels()
        # h2, l2 = ax2.get_legend_handles_labels()
        # plt.legend(handles=h1 + h2, labels=l1 + l2, loc=8, bbox_to_anchor=(0.5, -0.15), ncol=5)
        # ax1.set_ylim([0, 1.3])
        # ax1.yaxis.set_major_formatter(FuncFormatter(to_100percent))
        # ax1.set_xlabel('')
        # ax2.set_xlabel('')
        # ax1.set_ylabel('')
        # ax2.set_ylabel('')
        # plt.title('市场拥挤度')
        # plt.tight_layout()
        # sns.despine(top=True, right=False, left=False, bottom=False)
        # plt.savefig('{0}市场拥挤度.png'.format(self.data_path))
        return

    def StyleTiming(self):
        # 风格指数
        style_index = HBDB().read_index_daily_k_given_date_and_indexs(self.start_date, ['399370', '399371'])
        style_index = style_index[['zqdm', 'jyrq', 'spjg']]
        style_index.columns = ['INDEX_SYMBOL', 'TRADE_DATE', 'CLOSE_INDEX']
        style_index['TRADE_DATE'] = style_index['TRADE_DATE'].astype(str)
        style_index = style_index[style_index['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        style_index = style_index.pivot(index='TRADE_DATE', columns='INDEX_SYMBOL', values='CLOSE_INDEX').sort_index().reset_index()
        style_index = style_index[(style_index['TRADE_DATE'] > self.start_date) & (style_index['TRADE_DATE'] <= self.end_date)]
        style_index['TRADE_DATE'] = style_index['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(style_index['TRADE_DATE'].values, style_index['399370'].values, color=line_color_list[0], label='成长')
        plt.plot(style_index['TRADE_DATE'].values, style_index['399371'].values, color=line_color_list[1], label='价值')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=2)
        plt.title('风格指数', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '风格指数'))

        # 风格拥挤度
        style_crowding = FEDB().read_timing_data(['TRADE_DATE', 'GROWTH_CROWDING', 'VALUE_CROWDING'], 'timing_style', self.start_date, self.end_date)
        style_crowding['TRADE_DATE'] = style_crowding['TRADE_DATE'].astype(str)
        style_crowding = style_crowding[style_crowding['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        style_crowding = style_crowding.sort_values('TRADE_DATE')
        style_crowding = style_crowding[(style_crowding['TRADE_DATE'] > self.start_date) & (style_crowding['TRADE_DATE'] <= self.end_date)]
        style_crowding['TRADE_DATE'] = style_crowding['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(style_crowding['TRADE_DATE'].values, style_crowding['GROWTH_CROWDING'].values, color=line_color_list[0], label='成长')
        plt.plot(style_crowding['TRADE_DATE'].values, style_crowding['VALUE_CROWDING'].values, color=line_color_list[1], label='价值')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=2)
        plt.title('风格拥挤度', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '风格拥挤度'))

        # 规模拥挤度
        size_crowding = FEDB().read_timing_data(['TRADE_DATE', 'SIZE_CROWDING'], 'timing_style', self.start_date, self.end_date)
        size_crowding['TRADE_DATE'] = size_crowding['TRADE_DATE'].astype(str)
        size_crowding = size_crowding[size_crowding['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        size_crowding = size_crowding.sort_values('TRADE_DATE')
        size_crowding = size_crowding[(size_crowding['TRADE_DATE'] > self.start_date) & (size_crowding['TRADE_DATE'] <= self.end_date)]
        size_crowding['TRADE_DATE'] = size_crowding['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(size_crowding['TRADE_DATE'].values, size_crowding['SIZE_CROWDING'].values, color=line_color_list[0], label='规模')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=1)
        plt.title('规模拥挤度', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '规模拥挤度'))

        # 风格离散度
        style_spread = FEDB().read_timing_data(['TRADE_DATE', 'GROWTH_SPREAD', 'VALUE_SPREAD'], 'timing_style', self.start_date, self.end_date)
        style_spread['TRADE_DATE'] = style_spread['TRADE_DATE'].astype(str)
        style_spread = style_spread[style_spread['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        style_spread = style_spread.sort_values('TRADE_DATE')
        style_spread = style_spread[(style_spread['TRADE_DATE'] > self.start_date) & (style_spread['TRADE_DATE'] <= self.end_date)]
        style_spread['TRADE_DATE'] = style_spread['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(style_spread['TRADE_DATE'].values, style_spread['GROWTH_SPREAD'].values, color=line_color_list[0], label='成长')
        plt.plot(style_spread['TRADE_DATE'].values, style_spread['VALUE_SPREAD'].values, color=line_color_list[1], label='价值')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=2)
        plt.title('风格离散度', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '风格离散度'))

        # 规模离散度
        size_spread = FEDB().read_timing_data(['TRADE_DATE', 'SIZE_SPREAD'], 'timing_style', self.start_date, self.end_date)
        size_spread['TRADE_DATE'] = size_spread['TRADE_DATE'].astype(str)
        size_spread = size_spread[size_spread['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        size_spread = size_spread.sort_values('TRADE_DATE')
        size_spread = size_spread[(size_spread['TRADE_DATE'] > self.start_date) & (size_spread['TRADE_DATE'] <= self.end_date)]
        size_spread['TRADE_DATE'] = size_spread['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(size_spread['TRADE_DATE'].values, size_spread['SIZE_SPREAD'].values, color=line_color_list[0], label='规模')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=1)
        plt.title('规模离散度', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '规模离散度'))

        # 风格动量
        style_momentum = FEDB().read_timing_data(['TRADE_DATE', 'GROWTH_MOMENTUM', 'VALUE_MOMENTUM'], 'timing_style',  self.start_date, self.end_date)
        style_momentum['TRADE_DATE'] = style_momentum['TRADE_DATE'].astype(str)
        style_momentum = style_momentum[style_momentum['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        style_momentum = style_momentum.sort_values('TRADE_DATE')
        style_momentum['GROWTH_MOMENTUM'] = style_momentum['GROWTH_MOMENTUM'].rolling(250).apply(lambda x: (x.iloc[-1] - x.mean()) / x.std())
        style_momentum['VALUE_MOMENTUM'] = style_momentum['VALUE_MOMENTUM'].rolling(250).apply(lambda x: (x.iloc[-1] - x.mean()) / x.std())
        style_momentum = style_momentum[(style_momentum['TRADE_DATE'] > self.start_date) & (style_momentum['TRADE_DATE'] <= self.end_date)]
        style_momentum['TRADE_DATE'] = style_momentum['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(style_momentum['TRADE_DATE'].values, style_momentum['GROWTH_MOMENTUM'].values, color=line_color_list[0], label='成长')
        plt.plot(style_momentum['TRADE_DATE'].values, style_momentum['VALUE_MOMENTUM'].values, color=line_color_list[1], label='价值')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=2)
        plt.title('风格动量', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '风格动量'))

        # 规模动量
        size_momentum= FEDB().read_timing_data(['TRADE_DATE', 'SIZE_MOMENTUM'], 'timing_style', self.start_date, self.end_date)
        size_momentum['TRADE_DATE'] = size_momentum['TRADE_DATE'].astype(str)
        size_momentum = size_momentum[size_momentum['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        size_momentum = size_momentum.sort_values('TRADE_DATE')
        size_momentum['SIZE_MOMENTUM'] = size_momentum['SIZE_MOMENTUM'].rolling(250).apply(lambda x: (x.iloc[-1] - x.mean()) / x.std())
        size_momentum = size_momentum[(size_momentum['TRADE_DATE'] > self.start_date) & (size_momentum['TRADE_DATE'] <= self.end_date)]
        size_momentum['TRADE_DATE'] = size_momentum['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(size_momentum['TRADE_DATE'].values, size_momentum['SIZE_MOMENTUM'].values, color=line_color_list[0], label='规模')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=1)
        plt.title('规模动量', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '规模动量'))
        return

    def IndustryTiming(self, index_symbol):
        # 量能维度：成交额占比、换手率
        turnover = FEDB().read_timing_data(['TRADE_DATE', 'INDEX_SYMBOL', 'TURNOVER_PROPORTION', 'TURNOVER_RATE'], 'timing_industry', self.start_date, self.end_date)
        turnover = turnover[turnover['INDEX_SYMBOL'] == index_symbol]
        turnover['TRADE_DATE'] = turnover['TRADE_DATE'].astype(str)
        turnover = turnover[turnover['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        turnover = turnover.sort_values('TRADE_DATE')
        turnover['TRADE_DATE'] = turnover['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(turnover['TRADE_DATE'].values, turnover['TURNOVER_PROPORTION'].values, color=line_color_list[0],  label='成交额占比')
        plt.plot(turnover['TRADE_DATE'].values, turnover['TURNOVER_RATE'].values, color=line_color_list[1], label='换手率')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=2)
        plt.gca().yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.title('量能维度', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '量能维度'))

        # 量价相关系数
        corr = FEDB().read_timing_data(['TRADE_DATE', 'INDEX_SYMBOL', 'CORR'], 'timing_industry', self.start_date, self.end_date)
        corr = corr[corr['INDEX_SYMBOL'] == index_symbol]
        corr['TRADE_DATE'] = corr['TRADE_DATE'].astype(str)
        corr = corr[corr['TRADE_DATE'].isin(self.trade_df[self.trade_df['IS_WEEK_END'] == '1']['TRADE_DATE'].unique().tolist())]
        corr = corr.sort_values('TRADE_DATE')
        index = HBDB().read_index_daily_k_given_date_and_indexs(self.start_date, [index_symbol])
        index = index.rename(columns={'zqdm': 'INDEX_SYMBOL', 'jyrq': 'TRADE_DATE', 'spjg': 'CLOSE_INDEX'})
        index = index[['TRADE_DATE', 'INDEX_SYMBOL', 'CLOSE_INDEX']]
        index['TRADE_DATE'] = index['TRADE_DATE'].astype(str)
        index = index.sort_values('TRADE_DATE')
        corr = corr.merge(index, on=['TRADE_DATE', 'INDEX_SYMBOL'], how='left')
        corr['TRADE_DATE'] = corr['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax1 = plt.subplots(figsize=(12, 6))
        ax2 = ax1.twinx()
        ax1.plot(corr['TRADE_DATE'].values, corr['CORR'].values, color=line_color_list[0], label='量价相关系数')
        ax2.stackplot(corr['TRADE_DATE'].values, corr['CLOSE_INDEX'].values, color=line_color_list[3], alpha=0.5, labels=['指数走势（右轴）'])
        h1, l1 = ax1.get_legend_handles_labels()
        h2, l2 = ax2.get_legend_handles_labels()
        plt.legend(handles=h1 + h2, labels=l1 + l2, loc=8, bbox_to_anchor=(0.5, -0.15), ncol=2)
        ax1.set_ylim([-1, 1])
        ax2.set_ylim([2000, 7000])
        plt.title('量价相关系数', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=False, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '量价相关系数'))

        # 价格维度：创新高个股数量及占比、均线上个股数量及占比
        price = FEDB().read_timing_data(['TRADE_DATE', 'INDEX_SYMBOL', 'NEW_HIGH', 'NEW_HIGH_RATIO', 'MEAN_ABOVE', 'MEAN_ABOVE_RATIO'], 'timing_industry', self.start_date, self.end_date)
        price = price[price['INDEX_SYMBOL'] == index_symbol]
        price['TRADE_DATE'] = price['TRADE_DATE'].astype(str)
        price = price[price['TRADE_DATE'].isin(self.trade_df[self.trade_df['IS_MONTH_END'] == '1']['TRADE_DATE'].unique().tolist())]
        price = price.sort_values('TRADE_DATE')
        fig, ax1 = plt.subplots(figsize=(12, 6))
        ax2 = ax1.twinx()
        bar_width = 0.3
        ax1.bar(np.arange(len(price)) - 0.5 * bar_width, price['NEW_HIGH'].values, bar_width, label='创60个交易日新高个股数量', color=bar_color_list[0])
        ax1.bar(np.arange(len(price)) + 0.5 * bar_width, price['MEAN_ABOVE'].values, bar_width, label='20个交易日均线上个股数量', color=bar_color_list[7])
        ax2.plot(np.arange(len(price)), price['NEW_HIGH_RATIO'].values, label='创60个交易日新高个股占比（右轴）',  color=line_color_list[0])
        ax2.plot(np.arange(len(price)), price['MEAN_ABOVE_RATIO'].values, label='20个交易日均线上个股占比（右轴）', color=line_color_list[1])
        h1, l1 = ax1.get_legend_handles_labels()
        h2, l2 = ax2.get_legend_handles_labels()
        plt.legend(handles=h1 + h2, labels=l1 + l2, loc=8, bbox_to_anchor=(0.5, -0.25), ncol=4)
        ax1.set_xticks(np.arange(len(price)))
        ax1.set_xticklabels(labels=price['TRADE_DATE'].unique().tolist(), rotation=60)
        ax2.yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.title('价格维度', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=False, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '价格维度'))

        # 资金维度：主力资金净流入率、融资买入情绪
        cash = FEDB().read_timing_data(['TRADE_DATE', 'INDEX_SYMBOL', 'MAIN_CASH_PROPORTION', 'MARGIN_PROPORTION'], 'timing_industry', self.start_date, self.end_date)
        cash = cash[cash['INDEX_SYMBOL'] == index_symbol]
        cash['TRADE_DATE'] = cash['TRADE_DATE'].astype(str)
        cash = cash[cash['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        cash = cash.sort_values('TRADE_DATE')
        cash['TRADE_DATE'] = cash['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(cash['TRADE_DATE'].values, cash['MAIN_CASH_PROPORTION'].values, color=line_color_list[0], label='主力资金净流入率')
        plt.plot(cash['TRADE_DATE'].values, cash['MARGIN_PROPORTION'].values, color=line_color_list[1], label='融资买入情绪')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=2)
        plt.gca().yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.title('资金维度', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '资金维度'))

        # 分析师情绪：创新高个股数量及占比、均线上个股数量及占比
        consensus = FEDB().read_timing_data(['TRADE_DATE', 'INDEX_SYMBOL', 'CONSENSUS_UP', 'CONSENSUS_UP_RATIO', 'CONSENSUS_DOWN', 'CONSENSUS_DOWN_RATIO'], 'timing_industry', self.start_date, self.end_date)
        consensus = consensus[consensus['INDEX_SYMBOL'] == index_symbol]
        consensus['TRADE_DATE'] = consensus['TRADE_DATE'].astype(str)
        consensus = consensus[consensus['TRADE_DATE'].isin(self.trade_df[self.trade_df['IS_MONTH_END'] == '1']['TRADE_DATE'].unique().tolist())]
        consensus = consensus.sort_values('TRADE_DATE')
        fig, ax1 = plt.subplots(figsize=(12, 6))
        ax2 = ax1.twinx()
        bar_width = 0.3
        ax1.bar(np.arange(len(consensus)), consensus['CONSENSUS_UP'].values, bar_width, label='分析师一致预期上调个股数量', color=bar_color_list[0])
        # ax1.bar(np.arange(len(consensus)) + 0.5 * bar_width, consensus['CONSENSUS_DOWN'].values, bar_width, label='分析师一致预期下调个股数量', color=bar_color_list[7])
        ax2.plot(np.arange(len(consensus)), consensus['CONSENSUS_UP_RATIO'].values, label='分析师一致预期上调个股占比（右轴）', color=line_color_list[1])
        # ax2.plot(np.arange(len(consensus)), consensus['CONSENSUS_DOWN_RATIO'].values, label='分析师一致预期下调个股占比（右轴）', color=line_color_list[1])
        h1, l1 = ax1.get_legend_handles_labels()
        h2, l2 = ax2.get_legend_handles_labels()
        plt.legend(handles=h1 + h2, labels=l1 + l2, loc=8, bbox_to_anchor=(0.5, -0.25), ncol=4)
        ax1.set_xticks(np.arange(len(consensus)))
        ax1.set_xticklabels(labels=consensus['TRADE_DATE'].unique().tolist(), rotation=60)
        ax2.yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.title('分析师情绪', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=False, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '分析师情绪'))

        # 行业景气度
        prosperity = FEDB().read_timing_data(['TRADE_DATE', 'INDEX_SYMBOL', 'OPER_REVENUE_YOY_DIFF', 'NET_PROFIT_YOY_DIFF', 'ROE_TTM_DIFF'], 'timing_industry', self.start_date, self.end_date)
        prosperity = prosperity[prosperity['INDEX_SYMBOL'] == index_symbol]
        prosperity['TRADE_DATE'] = prosperity['TRADE_DATE'].astype(str)
        prosperity = prosperity[prosperity['TRADE_DATE'].isin(self.report_df['REPORT_DATE'].unique().tolist())]
        prosperity = prosperity.sort_values('TRADE_DATE')
        prosperity['PROSPERITY'] = (prosperity['OPER_REVENUE_YOY_DIFF'] + prosperity['NET_PROFIT_YOY_DIFF'] + prosperity['ROE_TTM_DIFF']) / 3.0
        fig, ax = plt.subplots(figsize=(12, 6))
        bar_width = 0.2
        ax.bar(np.arange(len(prosperity)), prosperity['PROSPERITY'].values, bar_width,  label='行业景气度', color=bar_color_list[0])
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.25), ncol=1)
        ax.set_xticks(np.arange(len(prosperity)))
        ax.set_xticklabels(labels=prosperity['TRADE_DATE'].unique().tolist(), rotation=60)
        ax.yaxis.set_major_formatter(FuncFormatter(to_percent))
        plt.title('行业景气度', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '行业景气度'))

        # 行业动量
        momentum = FEDB().read_timing_data(['TRADE_DATE', 'INDEX_SYMBOL', 'INDUSTRY_MOMENTUM'], 'timing_industry', self.start_date, self.end_date)
        momentum = momentum[momentum['INDEX_SYMBOL'] == index_symbol]
        momentum['TRADE_DATE'] = momentum['TRADE_DATE'].astype(str)
        momentum = momentum[momentum['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        momentum = momentum.sort_values('TRADE_DATE')
        momentum['TRADE_DATE'] = momentum['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.stackplot(momentum['TRADE_DATE'].values, momentum['INDUSTRY_MOMENTUM'].values, color=line_color_list[0], alpha=0.5, labels=['行业动量'])
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=2)
        plt.gca().yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.title('行业动量', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '行业动量'))
        return

    def StockTiming(self):
        # 成交额集中度
        turnover_proportion = FEDB().read_timing_data(['TRADE_DATE', 'TURNOVER_PROPORTION_TOP5', 'TURNOVER_PROPORTION_TOP10'], 'timing_stock', self.start_date, self.end_date)
        turnover_proportion['TRADE_DATE'] = turnover_proportion['TRADE_DATE'].astype(str)
        turnover_proportion = turnover_proportion[turnover_proportion['TRADE_DATE'].isin(self.trade_df['TRADE_DATE'].unique().tolist())]
        turnover_proportion = turnover_proportion.sort_values('TRADE_DATE')
        turnover_proportion['TRADE_DATE'] = turnover_proportion['TRADE_DATE'].apply(lambda x: datetime.strptime(x, '%Y%m%d'))
        fig, ax = plt.subplots(figsize=(12, 6))
        plt.plot(turnover_proportion['TRADE_DATE'].values, turnover_proportion['TURNOVER_PROPORTION_TOP5'].values, color=line_color_list[0], label='前5%个股成交额集中度')
        plt.plot(turnover_proportion['TRADE_DATE'].values, turnover_proportion['TURNOVER_PROPORTION_TOP10'].values, color=line_color_list[1], label='前10%个股成交额集中度')
        plt.legend(loc=8, bbox_to_anchor=(0.5, -0.15), ncol=2)
        plt.gca().yaxis.set_major_formatter(FuncFormatter(to_100percent))
        plt.title('成交额集中度', fontdict={'font': 'kaiti', 'weight': 'bold', 'size': 16})
        plt.tight_layout()
        sns.despine(top=True, right=True, left=False, bottom=False)
        plt.savefig('{0}{1}.png'.format(self.data_path, '成交额集中度'))
        return


if __name__ == '__main__':
    start_date = '20091231'
    end_date = '20230210'
    data_path = 'D:/Git/hbshare/hbshare/fe/xwq/data/timing/'
    TimingAnalysis(start_date, end_date, data_path).MarketTiming()
    TimingAnalysis(start_date, end_date, data_path).StyleTiming()
    TimingAnalysis(start_date, end_date, data_path).IndustryTiming('801010')
    TimingAnalysis(start_date, end_date, data_path).StockTiming()


"""
量化周报阿尔法数据更新
"""
import os
import numpy as np
import pandas as pd
import hbshare as hbs
import datetime
import math
from sqlalchemy import create_engine
from hbshare.quant.Kevin.quant_room.MyUtil.data_loader import get_fund_nav_from_sql, get_trading_day_list


fund_dict = {
    "300绝对": {"衍复": "SNP818"},
    "500绝对": {
        "天演": "SQP881",
        "诚奇": "SQK764",
        "明汯": "SEE194",
        "启林": "SNL641",
        "衍复": "SJH866",
        "黑翼": "SEM323",
        "星阔": "SNU706",
        "世纪前沿": "SGP682",
        "宽德": "SEJ470"},
    "1000绝对": {
        "九坤": "SCP381",
        "启林": "STE703",
        "衍复": "SJM688",
        "黑翼": "SQG281",
        "宽德": "STD264"
    },
    "量化多头": {
        "诚奇": "SSU249",
        "明汯": "SSL078",
        "星阔": "SSE288",
        "世纪前沿": "STS091",
    },
    "中性收益": {
        "诚奇": "SNR622"
    }
}


class WeeklyTracker:
    def __init__(self, start_date, pre_date, end_date):
        self.start_date = start_date
        self.pre_date = pre_date
        self.end_date = end_date

    def run(self):
        # TODO
        order_list = ['天演', '诚奇', '明汯', '九坤', '启林', '衍复', '黑翼', '星阔', '世纪前沿', '宽德']
        col_list = ['300绝对', '500绝对', '1000绝对', '量化多头', '中性收益']
        trading_day_list = get_trading_day_list(self.start_date, self.end_date, frequency="week")
        nav_list = []
        for key, value in fund_dict.items():
            nav_series = get_fund_nav_from_sql(self.start_date, self.end_date, value).reindex(trading_day_list)
            nav_series['trade_date'] = nav_series.index
            nav_series = nav_series.melt(id_vars='trade_date', var_name="fund_name", value_name="nav")
            nav_series['type'] = key
            nav_list.append(nav_series)
        nav_df = pd.concat(nav_list, axis=0)
        # 指数收益
        benchmark_list = ['000300', '000905', '000852']
        sql_script = "SELECT JYRQ as TRADEDATE, ZQDM as INDEXCODE, SPJG as TCLOSE from funddb.ZSJY WHERE ZQDM in ({}) " \
                     "and JYRQ >= {} and JYRQ <= {}".format(','.join("'{0}'".format(x) for x in benchmark_list),
                                                            self.start_date, self.end_date)
        res = hbs.db_data_query('readonly', sql_script, page_size=5000)
        map_dict = {"000905": "中证500", "000300": "沪深300", "000852": "中证1000"}
        index_data = pd.pivot_table(
            pd.DataFrame(res['data']), index='TRADEDATE', columns='INDEXCODE', values='TCLOSE').reindex(
            trading_day_list).rename(columns=map_dict)
        # 缺失值
        count_df = nav_df[['fund_name', 'type']].value_counts().to_frame('num').reset_index()
        count_df = pd.pivot_table(
            count_df, index='fund_name', columns='type', values='num').reindex(order_list)[col_list]
        print(count_df)
        # 单周表现
        t_df = pd.pivot_table(nav_df[nav_df['trade_date'] == self.end_date],
                              index='fund_name', columns='type', values='nav').reindex(order_list)[col_list]
        p_df = pd.pivot_table(nav_df[nav_df['trade_date'] == self.pre_date],
                              index='fund_name', columns='type', values='nav').reindex(order_list)[col_list]
        week_ret = t_df / p_df - 1
        week_ret['300超额'] = week_ret['300绝对'] - index_data.pct_change().loc[self.end_date, '沪深300']
        week_ret['500超额'] = week_ret['500绝对'] - index_data.pct_change().loc[self.end_date, '中证500']
        week_ret['1000超额'] = week_ret['1000绝对'] - index_data.pct_change().loc[self.end_date, '中证1000']
        col_list_out = ['300绝对', '300超额', '500绝对', '500超额', '1000绝对', '1000超额', '量化多头', '中性收益']
        week_ret = week_ret.applymap(lambda x: x if math.isnan(x) else format(x, '.2%'))[col_list_out]
        # 年初至今表现
        p_df = pd.pivot_table(nav_df[nav_df['trade_date'] == self.start_date],
                              index='fund_name', columns='type', values='nav').reindex(order_list)[col_list]
        period_ret = t_df / p_df - 1
        bm_period_ret = index_data.loc[self.end_date] / index_data.loc[self.start_date] - 1
        period_ret['300超额'] = period_ret['300绝对'] - bm_period_ret.loc['沪深300']
        period_ret['500超额'] = period_ret['500绝对'] - bm_period_ret.loc['中证500']
        period_ret['1000超额'] = period_ret['1000绝对'] - bm_period_ret.loc['中证1000']
        period_ret = period_ret.applymap(lambda x: x if math.isnan(x) else format(x, '.2%'))[col_list_out]

        excel_writer = pd.ExcelWriter('D:\\量化周报alpha数据_{}.xlsx'.format(self.end_date), engine='openpyxl')
        week_ret.to_excel(excel_writer, sheet_name='代销产品上周表现')
        period_ret.to_excel(excel_writer, sheet_name='代销产品今年以来表现')
        excel_writer.close()

        return week_ret, period_ret


if __name__ == '__main__':
    WeeklyTracker("20211231", "20221223", "20221230").run()


import pandas as pd
from hbshare.fe.XZ import db_engine
from hbshare.fe.XZ import functionality
from hbshare.fe.mutual_analysis import  holdind_based as hb
import numpy as np
from hbshare.fe.Machine_learning import classifier
import  datetime
import  joblib



util=functionality.Untils()
hbdb=db_engine.HBDB()
localdb=db_engine.PrvFunDB().engine


def get_pic_from_localdb(jjdm,asofdate='20220831',if_percentage=True,show_num=None):


    latest_date=pd.read_sql(
        "select max(asofdate) as asofdate from jjpic_value_p_hbs where asofdate<='{}'"
            .format(asofdate),con=localdb)['asofdate'][0]
    sql="SELECT * from jjpic_value_p_hbs where {0} and asofdate='{1}' "\
        .format(jjdm,latest_date)
    value_p_hbs=pd.read_sql(sql,con=localdb)

    latest_date=pd.read_sql(
        "select max(asofdate) as asofdate from jjpic_size_p_hbs where asofdate<='{}'"
            .format(asofdate),con=localdb)['asofdate'][0]
    sql="SELECT * from jjpic_size_p_hbs where {0} and asofdate='{1}' "\
        .format(jjdm,latest_date)
    size_p_hbs=pd.read_sql(sql,con=localdb).drop('规模偏好',axis=1)


    latest_date=pd.read_sql(
        "select max(asofdate) as asofdate from jjpic_industry_p where asofdate<='{}'"
            .format(asofdate),con=localdb)['asofdate'][0]
    ind_max_date=latest_date
    sql="SELECT * from jjpic_industry_p where {0} and asofdate='{1}' "\
        .format(jjdm,latest_date)
    industry_p=pd.read_sql(sql,con=localdb)

    float_col_list=industry_p.columns.tolist()
    float_col_list.remove('jjdm')
    float_col_list.remove('asofdate')
    float_col_list.remove('前五大行业')
    float_col_list.remove('二级行业前20大')
    float_col_list.remove('三级行业前20大')
    float_col_list.remove('基金简称')
    float_col_list.remove('一级行业类型')
    float_col_list.remove('二级行业类型')
    float_col_list.remove('三级行业类型')


    latest_date=pd.read_sql(
        "select max(asofdate) as asofdate from jjpic_theme_p where asofdate<='{}'"
            .format(asofdate),con=localdb)['asofdate'][0]
    sql="SELECT * from jjpic_theme_p where {0} and asofdate='{1}' "\
        .format(jjdm,latest_date)
    theme_p=pd.read_sql(sql,con=localdb)


    latest_date=pd.read_sql(
        "select max(asofdate) as asofdate from jjpic_industry_sp where asofdate<='{}'"
            .format(asofdate),con=localdb)['asofdate'][0]
    sql="SELECT * from jjpic_industry_sp where {0} and asofdate='{1}' "\
        .format(jjdm,latest_date)
    industry_sp=pd.read_sql(sql,con=localdb)
    ind_sp_float_col_list=industry_sp.columns.tolist()
    ind_sp_float_col_list.remove('jjdm')
    ind_sp_float_col_list.remove('asofdate')
    ind_sp_float_col_list.remove('项目名')

    latest_date=pd.read_sql(
        "select max(asofdate) as asofdate from jjpic_theme_sp where asofdate<='{}'"
            .format(asofdate),con=localdb)['asofdate'][0]
    sql="SELECT * from jjpic_theme_sp where {0} and asofdate='{1}' "\
        .format(jjdm,latest_date)
    theme_sp=pd.read_sql(sql,con=localdb)
    theme_sp_float_col_list = theme_sp.columns.tolist()
    theme_sp_float_col_list.remove('jjdm')
    theme_sp_float_col_list.remove('asofdate')
    theme_sp_float_col_list.remove('项目名')

    latest_date=pd.read_sql(
        "select max(asofdate) as asofdate from jjpic_stock_p where asofdate<='{}'"
            .format(asofdate),con=localdb)['asofdate'][0]
    sql="SELECT * from jjpic_stock_p where {0} and asofdate='{1}' "\
        .format(jjdm,latest_date)
    stock_p=pd.read_sql(sql,con=localdb)
    float_col=stock_p.columns.tolist()
    float_col.remove('jjdm')
    float_col.remove('asofdate')
    float_col.remove('持股数量')
    float_col.remove('PE')
    float_col.remove('PB')
    float_col.remove('ROE')
    float_col.remove('股息率')
    float_col.remove('PE_中位数')
    float_col.remove('PB_中位数')
    float_col.remove('ROE_中位数')
    float_col.remove('股息率_中位数')
    float_col.remove('基金简称')
    float_col.remove('个股风格A')
    float_col.remove('个股风格B')
    float_col.remove('是否有尾仓(针对高个股集中基金)')

    latest_date=pd.read_sql(
        "select max(asofdate) as asofdate from jjpic_stock_tp where asofdate<='{}'"
            .format(asofdate),con=localdb)['asofdate'][0]
    sql="SELECT * from jjpic_stock_tp where {0} and asofdate='{1}' "\
        .format(jjdm,latest_date)
    stock_tp=pd.read_sql(sql,con=localdb)
    tp_float_col=stock_tp.columns.tolist()
    tp_float_col.remove('jjdm')
    tp_float_col.remove('平均持有时间(出重仓前)')
    tp_float_col.remove('平均持有时间(出持仓前)')
    tp_float_col.remove('asofdate')
    tp_float_col.remove('基金简称')
    tp_float_col.remove('左侧标签')
    tp_float_col.remove('新股次新股偏好')

    #update the industry label by ticker shift ratio
    industry_p = pd.merge(industry_p, stock_tp[['jjdm', '换手率_rank']],how='left',on='jjdm')
    industry_p.loc[(industry_p['一级行业类型']=='博弈')
                   &(industry_p['换手率_rank']<0.5),'一级行业类型']='博弈(被动)'
    industry_p.loc[(industry_p['一级行业类型']=='轮动')
                   &(industry_p['换手率_rank']<0.5),'一级行业类型']='轮动(被动)'
    industry_p.drop('换手率_rank',axis=1,inplace=True)

    # latest_date=pd.read_sql(
    #     "select max(asofdate) as asofdate from nav_jj_ret_analysis where  asofdate<='{}'"
    #         .format(asofdate),con=localdb)['asofdate'][0]
    # sql="SELECT * from nav_jj_ret_analysis where {0} and asofdate='{1}' "\
    #     .format(jjdm,latest_date)
    # jj_performance=pd.read_sql(sql,con=localdb).fillna(0)
    #
    # latest_date=pd.read_sql(
    #     "select max(asofdate) as asofdate from nav_hurst_index where  asofdate<='{}'"
    #         .format(asofdate),con=localdb)['asofdate'][0]
    # sql="SELECT * from nav_hurst_index where {0} and asofdate='{1}' "\
    #     .format(jjdm,latest_date)
    # hurst=pd.read_sql(sql,con=localdb)
    # hurst['业绩特征'] = '随机'
    # hurst.loc[hurst['H'] <= 0.35,'业绩特征']='趋势反转'
    # hurst.loc[hurst['H'] >= 0.65,'业绩特征']='趋势保持'
    #
    # jj_performance=pd.merge(jj_performance,hurst[['jjdm','业绩特征']],how='left',on='jjdm')

    # jj_performance=jj_performance[['jjdm','业绩预期回归强度', '长期相对业绩表现', '相对业绩稳定性','业绩特征','month_rank_mean', 'month_rank_std', 'quarter_rank_mean',
    #    'quarter_rank_std', 'hy_rank_mean', 'hy_rank_std', 'yearly_rank_mean',
    #    'yearly_rank_std', 'ret_regress',
    #    'sharp_ratio', 'sharp_ratio_rank', 'downwards_ratio',
    #    'downwards_ratio_rank', 'sortino_ratio', 'sortino_ratio_rank',
    #    'max_drawback', 'max_drawback_rank', 'calmark_ratio',
    #    'calmark_ratio_rank', 'treynor_ratio', 'treynor_ratio_rank','moment','moment_rank',
    #    'asofdate']]
    # performance_float_col=jj_performance.columns.tolist()
    # performance_float_col.remove('jjdm')
    # performance_float_col.remove('业绩预期回归强度')
    # performance_float_col.remove('长期相对业绩表现')
    # performance_float_col.remove('相对业绩稳定性')
    # performance_float_col.remove('asofdate')
    # performance_float_col.remove('业绩特征')


    if (if_percentage):
        for col in float_col_list:
            industry_p[col] = industry_p[col].map("{:.2%}".format)
        for col in ind_sp_float_col_list[0:int(len(ind_sp_float_col_list) / 2)]:
            industry_sp[col] = \
                industry_sp[col].astype(float).map("{:.2%}".format)
        for col in ind_sp_float_col_list[int(len(ind_sp_float_col_list) / 2):]:
            industry_sp.loc[industry_sp['项目名'] != '切换次数', col] = \
                industry_sp.iloc[1:][col].astype(float).map("{:.2%}".format)
        for col in theme_sp_float_col_list[0:int(len(theme_sp_float_col_list) / 2)]:
            theme_sp[col] = \
                theme_sp[col].astype(float).map("{:.2%}".format)
        for col in theme_sp_float_col_list[int(len(theme_sp_float_col_list) / 2):]:
            theme_sp.loc[theme_sp['项目名'] != '切换次数', col] = \
                theme_sp.iloc[1:][col].astype(float).map("{:.2%}".format)
        for col in ['主题集中度', '主题换手率', '大金融', '消费', 'TMT', '周期',
       '制造']:
            theme_p[col]=theme_p[col].map("{:.2%}".format)


        for col in ['集中度(持仓)', '换手率(持仓)', '成长绝对暴露(持仓)', '价值绝对暴露(持仓)', '集中度排名(持仓)',
           '换手率排名(持仓)', '成长暴露排名(持仓)', '价值暴露排名(持仓)']:
            value_p_hbs[col]=value_p_hbs[col].map("{:.2%}".format)
        for col in ['集中度(持仓)', '换手率(持仓)', '大盘绝对暴露(持仓)', '中盘绝对暴露(持仓)', '小盘绝对暴露(持仓)',
           '集中度排名(持仓)', '换手率排名(持仓)', '大盘暴露排名(持仓)', '中盘暴露排名(持仓)', '小盘暴露排名(持仓)']:
            size_p_hbs[col]=size_p_hbs[col].map("{:.2%}".format)


        for col in float_col:
            stock_p[col]= stock_p[col].map("{:.2%}".format)
        for col in tp_float_col:
            stock_tp[col] = stock_tp[col].map("{:.2%}".format)

        # for col in performance_float_col:
        #     jj_performance[col] = jj_performance[col].map("{:.2%}".format)

    return value_p_hbs,size_p_hbs,\
           industry_p,industry_sp,theme_p,theme_sp,stock_p,\
           stock_tp

def get_fund_features(asofdate,asofdate2,jjdm_list):


    if(jjdm_list is not None):
        jjdm_con="jjdm in ({})".format(util.list_sql_condition(jjdm_list))
    else:
        jjdm_con='1=1'

    value_p_hbs,size_p_hbs,  \
    industry_p, industry_sp, theme_p, theme_sp, stock_p, stock_tp= \
        get_pic_from_localdb(jjdm_con, asofdate=asofdate,if_percentage=False)

    jj_features=pd.merge(value_p_hbs.drop(['风格偏好','成长绝对暴露(持仓)',
                                           '价值绝对暴露(持仓)','成长暴露排名(持仓)', '价值暴露排名(持仓)'],axis=1)
                         ,size_p_hbs.drop(['asofdate','基金简称','大盘绝对暴露(持仓)','中盘绝对暴露(持仓)',
                                           '小盘绝对暴露(持仓)', '大盘暴露排名(持仓)','中盘暴露排名(持仓)', '小盘暴露排名(持仓)'],axis=1)
                         ,how='inner',on='jjdm')

    jj_features=pd.merge(jj_features,industry_p.drop(['asofdate','基金简称','前五大行业','二级行业前20大','三级行业前20大'],axis=1)
                         ,how='inner',on='jjdm')
    jj_features=pd.merge(jj_features,
                         industry_sp[['项目名','Total_rank','jjdm']].pivot_table('Total_rank','jjdm','项目名')
                         ,how='left',on='jjdm')
    jj_features=pd.merge(jj_features,theme_p.drop(['asofdate','基金简称','大金融','消费','TMT','周期','制造'],axis=1)
                         ,how='inner',on='jjdm')
    jj_features=pd.merge(jj_features,stock_p.drop(['asofdate','基金简称'],axis=1)
                         ,how='inner',on='jjdm')
    jj_features=pd.merge(jj_features,stock_tp.drop(['asofdate','基金简称'],axis=1)
                         ,how='inner',on='jjdm')

    objective_col=jj_features.dtypes[jj_features.dtypes=='O'].index.tolist()

    objective_col.remove('jjdm')
    objective_col.remove('基金简称')
    objective_col.remove('asofdate')
    dummpy_data=pd.get_dummies(jj_features[objective_col])

    jj_features=pd.concat([jj_features.drop(objective_col,axis=1),dummpy_data],axis=1)


    if(asofdate2[4:6]=='12'):
        return_date=str(int(asofdate2[0:4])+1)+'0930'
        # return_date = str(int(asofdate2[0:4]) + 2) + '0330'
        start_date=str(int(asofdate2[0:4])+1)+'0331'
    else:
        return_date =str(int(asofdate2[0:4])+1) + '0331'
        # return_date = str(int(asofdate2[0:4]) + 1) + '0930'
        start_date = asofdate2[0:4] + '0930'

    sql="select jjdm,zbnp from st_fund.t_st_gm_rqjhb where jjdm in ({0}) and zblb='2106' and jzrq>='{1}' and jzrq<='{2}'"\
        .format(util.list_sql_condition(jjdm_list),return_date[0:6]+'15',return_date[0:6]+'31')
    jj_half_year_ret=hbdb.db2df(sql,db='funduser').drop_duplicates('jjdm',keep='last')

    # sql="select jjdm,hb1n from st_fund.t_st_gm_nhb where tjnf={0} and jjdm in ({1})"\
    #     .format(str(int(asofdate[0:4])+1),util.list_sql_condition(jjdm_list))
    # jj_year_ret=hbdb.db2df(sql,db='funduser')

    ratio=0.9

    jj_features=jj_features[((jj_features['一级行业集中度']>0.8)
                            &(jj_features['一级行业换手率'])<0.2)]

    jj_features=pd.merge(jj_features,jj_half_year_ret,how='left',on='jjdm')
    jj_features['zbnp']=jj_features['zbnp'].rank(method='min')/len(jj_features)
    jj_features['flag']=1
    jj_features.loc[jj_features['zbnp']>=ratio,'flag']=0

    # jj_features=jj_features[((jj_features['zbnp']>ratio)|(jj_features['zbnp']<1-ratio))
    #                         |(jj_features['asofdate']=='20210630')]

    jj_features.drop('zbnp',axis=1,inplace=True)


    sql="select jjdm,zbnp,zblb from st_fund.t_st_gm_zqjxxbl where tjrq>={0} and tjrq<={1} and jjdm in ({2}) and zblb in ('2201','2202','2203','2106','2103','2101') and zbnp!=99999"\
        .format(start_date[0:6]+'01',start_date[0:6]+'31',util.list_sql_condition(jj_features['jjdm'].unique().tolist()))
    info_ratio=hbdb.db2df(sql,db='funduser').drop_duplicates(['jjdm','zblb'],keep='last').pivot_table('zbnp','jjdm','zblb')

    float_col=jj_features.dtypes[jj_features.dtypes!='O'].index.tolist()
    jj_features[float_col]=jj_features[float_col].fillna(0)

    jj_features=pd.merge(jj_features,info_ratio,how='left',on='jjdm')


    return jj_features


if __name__ == '__main__':

    jj_features=[]
    for asofdate in ['20181228','20190628','20191231'
        , '20200630',  '20201231'
        , '20210630']:
        print(asofdate)
        jjdm_list=util.get_bmk_funds_list(asofdate)

        jj_features.append(get_fund_features(asofdate=(datetime.datetime.strptime(asofdate, '%Y%m%d')
                                     +datetime.timedelta(days=120)).strftime('%Y%m%d'),asofdate2=asofdate, jjdm_list=jjdm_list))

    jj_features=pd.concat(jj_features,axis=0)
    jj_features.to_excel('mchl_data.xlsx',index=False)
    print('20210630')
    test_ratio=len(jj_features[jj_features['asofdate']<='20201231'])-1
    model, f1_score=\
        classifier.model_built_up(jj_features.drop(['jjdm','基金简称','asofdate'],axis=1).fillna(0),'flag','randomforest',[],test_ratio)

    # model, f1_score = \
    #     classifier.model_built_up(jj_features.drop(['jjdm', '基金简称', 'asofdate'], axis=1).fillna(0), 'flag', 'xgboost',
    #                               [], test_ratio)
    # joblib.dump(model,'xgboost_20210630.pkl')

    # feature_imp = pd.DataFrame(data=model.feature_importances_,
    #                            index=jj_features.drop(['jjdm', '基金简称', 'asofdate', 'flag'], axis=1).columns)
    # print(feature_imp.sort_values(0, ascending=False).iloc[0:10])
    #
    # #
    # print('20201231')
    # jj_features=jj_features[jj_features['asofdate']<='20201231']
    # test_ratio = len(jj_features[jj_features['asofdate'] <= '20200630']) - 1
    # model, f1_score = \
    #     classifier.model_built_up(jj_features.drop(['jjdm', '基金简称', 'asofdate'], axis=1).fillna(0), 'flag',
    #                               'xgboost', [], test_ratio)
    # joblib.dump(model,'xgboost_20201231.pkl')
    # feature_imp = pd.DataFrame(data=model.feature_importances_,
    #                            index=jj_features.drop(['jjdm', '基金简称', 'asofdate', 'flag'], axis=1).columns)
    # print(feature_imp.sort_values(0, ascending=False).iloc[0:10])
    # #
    # print('20200630')
    # jj_features=jj_features[jj_features['asofdate']<='20200630']
    # test_ratio = len(jj_features[jj_features['asofdate'] <= '20191231']) - 1
    # model, f1_score = \
    #     classifier.model_built_up(jj_features.drop(['jjdm', '基金简称', 'asofdate'], axis=1).fillna(0), 'flag',
    #                               'xgboost', [], test_ratio)
    # joblib.dump(model, 'xgboost_20200630.pkl')
    # feature_imp = pd.DataFrame(data=model.feature_importances_,
    #                            index=jj_features.drop(['jjdm', '基金简称', 'asofdate', 'flag'], axis=1).columns)
    # print(feature_imp.sort_values(0, ascending=False).iloc[0:10])
    # #
    # print('20191231')
    # jj_features=jj_features[jj_features['asofdate']<='20191231']
    # test_ratio = len(jj_features[jj_features['asofdate'] <= '20190628']) - 1
    # model, f1_score = \
    #     classifier.model_built_up(jj_features.drop(['jjdm', '基金简称', 'asofdate'], axis=1).fillna(0), 'flag',
    #                               'xgboost', [], test_ratio)
    # joblib.dump(model, 'xgboost_20191231.pkl')
    # feature_imp = pd.DataFrame(data=model.feature_importances_,
    #                            index=jj_features.drop(['jjdm', '基金简称', 'asofdate', 'flag'], axis=1).columns)
    # print(feature_imp.sort_values(0, ascending=False).iloc[0:10])

    # #
    # jj_features=pd.read_excel('mchl_data.xlsx')
    # asofdate = '20211231'
    # jjdm_list = util.get_bmk_funds_list(asofdate)
    # test = get_fund_features(asofdate=(datetime.datetime.strptime(asofdate, '%Y%m%d')
    #                                  +datetime.timedelta(days=120)).strftime('%Y%m%d'),asofdate2=asofdate, jjdm_list=jjdm_list)
    # test = pd.concat([jj_features.iloc[[-1]], test], axis=0)
    # #
    # voting_result=[]
    # fi_summary=pd.DataFrame(columns=['index'])
    # for asofdate in ['20191231'
    #     , '20200630',  '20201231'
    #     , '20210630']:
    #     #to make sure the columns is the same
    #     print(asofdate)
    #     model=joblib.load('xgboost_{}.pkl'.format(asofdate))
    #     feature_imp = pd.DataFrame(data=model.feature_importances_,
    #                                index=jj_features.drop(['jjdm', '基金简称', 'asofdate', 'flag'], axis=1).columns)
    #     fi_summary=\
    #         pd.merge(fi_summary, feature_imp.reset_index().rename(columns={0:asofdate}), how='outer', on='index')
    #
    #     result = model.predict(test.drop(['jjdm', '基金简称', 'asofdate', 'flag'], axis=1).fillna(0))
    #     test['pred']=result
    #     test.loc[test['pred']<0.5,'pred']=0
    #     test.loc[test['pred'] > 0.5, 'pred'] = 1
    #     print('{}'.format(len(test[(test['pred']==0)&(test['flag']==0)])/len(test[test['pred']==0])))
    #
    #
    #     voting_result.append(test[test['pred']<0.5]['jjdm'])
    #     test.drop('pred',axis=1,inplace=True)
    #
    # fi_summary.set_index('index',inplace=True)
    #
    # print('')





# -*- coding: utf-8 -*-

import pandas as pd
import pymysql
import hbshare as hbs
import warnings
warnings.filterwarnings('ignore', category=pymysql.Warning)


class Loader:
    def __init__(self):
        pass

    def get_df(self, sql, db, page_size=2000):
        data = hbs.db_data_query(db, sql, page_size=page_size, timeout=120)
        pages = data['pages']
        data = pd.DataFrame(data['data'])
        if pages > 1:
            for page in range(2, pages + 1):
                temp_data = hbs.db_data_query(db, sql, page_size=page_size, page_num=page, timeout=120)
                data = pd.concat([data, pd.DataFrame(temp_data['data'])], axis=0)
        return data

    def read_cal(self, start, end):
        sql = "SELECT jyrq, sfjj, sfzm, sfym FROM st_main.t_st_gg_jyrl WHERE jyrq>={0} and jyrq<={1}".format(start, end)
        df = self.get_df(sql, db='alluser')
        return df

    def read_mutual_index_daily_k_given_indexs(self, indexs, start, end):
        indexs = "'" + "','".join(indexs) + "'"
        sql = "SELECT zsdm AS INDEX_CODE, jzrq AS TRADE_DATE, spjg AS CLOSE_INDEX FROM st_fund.t_st_gm_clzs WHERE zsdm in ({0}) AND jzrq >= {1} AND jzrq <= {2} AND m_opt_type <> '03'".format(indexs, start, end)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_private_index_daily_k_given_indexs(self, indexs, start, end):
        indexs = "'" + "','".join(indexs) + "'"
        sql = "SELECT zsdm AS INDEX_CODE, jyrq AS TRADE_DATE, spjg AS CLOSE_INDEX FROM st_hedge.t_st_sm_zhmzs WHERE zsdm in ({0}) AND jyrq >= {1} AND jyrq <= {2} AND m_opt_type <> '03'".format(indexs, start, end)
        df = self.get_df(sql, db='highuser', page_size=200000)
        return df

    def read_market_index_daily_k_given_indexs(self, indexs, start, end):
        indexs = "'" + "','".join(indexs) + "'"
        sql = "SELECT zqdm AS INDEX_CODE, jyrq AS TRADE_DATE, spjg AS CLOSE_INDEX FROM st_market.t_st_zs_hqql WHERE zqdm in ({0}) AND jyrq >= {1} AND jyrq <= {2} AND m_opt_type <> '03'".format(indexs, start, end)
        df = self.get_df(sql, db='alluser', page_size=200000)
        return df

    def read_mutual_fund_cumret_given_codes(self, codes, start, end):
        codes = "'" + "','".join(codes) + "'"
        sql = "SELECT jjdm AS FUND_CODE, jzrq AS TRADE_DATE, fqdwjz AS ADJ_NAV FROM st_fund.t_st_gm_rhb WHERE jjdm in ({0}) AND jzrq >= {1} AND jzrq <= {2} AND m_opt_type <> '03'".format(codes, start, end)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df

    def read_private_fund_adj_nav_given_codes(self, codes, start, end):
        codes = "'" + "','".join(codes) + "'"
        sql = "SELECT jjdm AS FUND_CODE, jzrq AS TRADE_DATE, fqdwjz AS ADJ_NAV FROM st_hedge.t_st_rhb WHERE jjdm in ({0}) AND jzrq >= {1} AND jzrq <= {2} AND m_opt_type <> '03'".format(codes, start, end)
        df = self.get_df(sql, db='highuser', page_size=200000)
        return df

    def read_market_index_market_value_given_indexs(self, indexs, start, end):
        indexs = "'" + "','".join(indexs) + "'"
        sql = "SELECT zqdm AS INDEX_CODE, jyrq AS TRADE_DATE, zsz AS MARKET_VALUE FROM st_market.t_st_zs_hqql WHERE zqdm in ({0}) AND jyrq >= {1} AND jyrq <= {2} AND m_opt_type <> '03'".format(indexs, start, end)
        df = self.get_df(sql, db='alluser', page_size=200000)
        return df

    def read_mutual_fund_market_value_given_codes(self, codes, start, end):
        codes = "'" + "','".join(codes) + "'"
        sql = "SELECT jjdm AS FUND_CODE, jsrq AS REPORT_DATE, jjzzc AS MARKET_VALUE FROM st_fund.t_st_gm_zcpz WHERE jjdm in ({0}) AND jsrq >= {1} AND jsrq <= {2} AND m_opt_type <> '03'".format(codes, start, end)
        df = self.get_df(sql, db='funduser', page_size=200000)
        return df


if __name__ == '__main__':
    import time
    import pandas as pd

    df = pd.read_csv('D:/Git/hbshare/hbshare/fe/xwq/data/fund.csv')
    df['FUND_CODE'] = df['FUND_CODE'].apply(lambda x: str(x).zfill(6))
    codes = df['FUND_CODE'].unique().tolist()[:40]
    t1 = time.time()
    data = Loader().read_mutual_fund_cumret_given_codes(codes, '20121027', '20221027')
    t2 = time.time()
    print(t2 - t1)
    data = data.pivot(index='TRADE_DATE', columns='FUND_CODE', values='ADJ_NAV')
    pass

    df = pd.read_csv('D:/Git/hbshare/hbshare/fe/xwq/data/index_aam.csv')
    df['INDUSTRY_ID'] = df['INDUSTRY_ID'].apply(lambda x: str(x).zfill(6))
    indexs = df['INDUSTRY_ID'].unique().tolist() + ['000300', '000905', '000906', 'CBA00301', '000852', '399001', '885001', '881001', '399006']
    t3 = time.time()
    data = Loader().read_market_index_daily_k_given_indexs(indexs, '20171027', '20221027')
    t4 = time.time()
    print(t4 - t3)
    data = data.pivot(index='TRADE_DATE', columns='INDEX_CODE', values='CLOSE_INDEX')
    pass


"""
数据准备模块
"""
import pandas as pd
import hbshare as hbs
import numpy as np
from datetime import datetime
from hbshare.quant.Kevin.quant_room.MyUtil.data_loader import get_fund_nav_from_sql, get_trading_day_list
from Arbitrage_backtest import cal_annual_return, cal_annual_volatility, cal_sharpe_ratio, cal_max_drawdown
from sqlalchemy import create_engine
from hbshare.quant.Kevin.rm_associated.config import engine_params
from tqdm import tqdm
from WindPy import w
import matplotlib.pyplot as plt

plt.style.use('seaborn')

w.start()


def load_benchmark(start_date, end_date, benchmark_id):
    sql_script = "SELECT JYRQ as TRADEDATE, ZQDM, SPJG as TCLOSE from funddb.ZSJY WHERE" \
                 " ZQDM = '{}' " \
                 "and JYRQ >= {} and JYRQ <= {}".format(benchmark_id, start_date, end_date)
    res = hbs.db_data_query('readonly', sql_script, page_size=5000)
    data = pd.DataFrame(res['data'])
    benchmark_df = data.set_index('TRADEDATE')['TCLOSE']

    return benchmark_df


def data_preparation(start_date, end_date):
    # db
    benchmark_id_list = ['000300', '000905', '000852', '000001', 'CBA00201']
    trading_day_list = get_trading_day_list(start_date, end_date, frequency="week")
    benchmark_list = []
    for benchmark_id in benchmark_id_list:
        benchmark_series = load_benchmark(start_date, end_date, benchmark_id).reindex(
            trading_day_list).to_frame(benchmark_id)
        benchmark_list.append(benchmark_series)
    benchmark_df = pd.concat(benchmark_list, axis=1)
    # 好买策略指数
    sql_script = "SELECT zsdm, spjg, jyrq FROM st_hedge.t_st_sm_zhmzs WHERE " \
                 "zsdm in ('HB1002','HB0018','HB0015','HB0017') and jyrq <= '20221230'"
    res = hbs.db_data_query('highuser', sql_script, page_size=5000)
    data = pd.DataFrame(res['data'])
    hb_index = pd.pivot_table(
        data, index='jyrq', columns='zsdm', values='spjg').reindex(trading_day_list).dropna(how='all').loc["20151231":]
    # strategy
    res = w.wsd(
        "NH0100.NHF,885001.WI,885306.WI,885309.WI,885308.WI,885312.WI", "close", start_date, end_date, "Period=W")
    d_list = [datetime.strftime(x, '%Y%m%d') for x in res.Times]
    data = pd.DataFrame(res.Data, index=res.Codes, columns=d_list).T.reindex(trading_day_list)
    benchmark_df = benchmark_df.merge(data[['NH0100.NHF']], left_index=True, right_index=True)
    strategy_index = data[data.columns[1:]].dropna()

    # 计算
    benchmark_df.pct_change().dropna(how='all').apply(lambda x: cal_sharpe_ratio(x, 0.015), axis=0)
    strategy_index.pct_change().dropna(how='all').apply(lambda x: cal_sharpe_ratio(x, 0.015), axis=0)

    return hb_index


def get_data_from_Wind():
    trading_day_list = get_trading_day_list('20150101', '20221231', frequency="day")
    # 主力资金数据
    date_list = trading_day_list[::40] + [trading_day_list[-1]]
    data_list = []
    for i in tqdm(range(1, len(date_list))):
        start_date, end_date = date_list[i - 1], date_list[i]
        res = w.wset("marketmoneyflows", "startdate={};enddate={};frequency=day;sector=sse_szse;securitytype=A股;field=date,maininmoney,mainoutmoney,maininflowmoney".format(start_date, end_date))
        data = pd.DataFrame(
            {"trade_date": res.Data[0], "M_in": res.Data[1], "M_out": res.Data[2], "Delta": res.Data[3]})
        data_list.append(data)
    all_data = pd.concat(data_list, axis=0)
    all_data['trade_date'] = all_data['trade_date'].apply(lambda x: datetime.strftime(x, "%Y%m%d"))
    all_data = all_data.drop_duplicates(subset=['trade_date']).set_index('trade_date').sort_index()
    all_data['sum'] = all_data['M_in'] + all_data['M_out']
    all_data = all_data.reindex(trading_day_list).dropna()
    # 成交额数据
    sql_script = "SELECT * FROM mac_stock_trading"
    engine = create_engine(engine_params)
    data = pd.read_sql(sql_script, engine)
    data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
    amt_data = data[['trade_date', 'amt_sh', 'amt_sz', 'amt_300', 'amt_500', 'amt_1000', 'amt_other']]
    amt_data['amt_all'] = amt_data['amt_sh'] + amt_data['amt_sz']
    amt_data = amt_data[(amt_data['trade_date'] > '20141231') & (amt_data['trade_date'] <= '20221231')]
    amt_data = amt_data.set_index('trade_date')[['amt_all']]

    df = all_data[['sum']].merge(amt_data, left_index=True, right_index=True)
    df['sum'] /= 1e+4

    # 按月度分类
    month_end = get_trading_day_list('20141220', '20221231', frequency="month")[::3]
    a = []
    b = []
    for i in range(1, len(month_end)):
        start, end = month_end[i - 1], month_end[i]
        period_data = df.loc[start: end][1:]
        ratio = period_data['sum'].sum() / period_data['amt_all'].sum()
        # ratio = period_data['sum'].sum()
        a.append(end)
        b.append(ratio)

    count_df = pd.DataFrame({"date": a, "ratio": b}).sort_values(by='date')
    count_df.set_index('date').plot.bar()


def curve_fitting(start_date, end_date, tday):
    """
    新方程指增FOF历史业绩拟合
    """
    config_df = pd.read_excel(
        "D:\\研究基地\\私募量化基金池\\量化基金池202302.xlsx", sheet_name="量化池列表", header=1).dropna(subset=["基金管理人"])
    config_df = config_df[(config_df['二级策略'] == '500指数增强') & (config_df['三级策略'] != "T0")]
    config_df['入池时间'] = config_df['入池时间'].apply(lambda x: datetime.strftime(x, "%Y%m%d"))
    # 微调
    config_df.loc[config_df['基金管理人'] == "幻方量化", ["代表产品", "基金代码"]] = ["九章幻方中证500量化进取1号", "SEC185"]
    config_df.loc[config_df['基金管理人'] == "诚奇资产", ["代表产品", "基金代码"]] = ["诚奇中证500增强精选1期", "SLS817"]
    config_df.loc[config_df['基金管理人'] == "天演资本", ["代表产品", "基金代码"]] = ["天演中证500指数增强", "P20830"]
    config_df.loc[config_df['基金管理人'] == "明汯投资", ["代表产品", "基金代码"]] = ["明汯价值成长1期", "SS5789"]
    fund_dict = config_df.set_index('代表产品')['基金代码'].to_dict()
    trading_day_list = get_trading_day_list(start_date, end_date, "week")
    fund_nav = get_fund_nav_from_sql(start_date, end_date, fund_dict).reindex(
        trading_day_list)
    # 入池矩阵
    fund_list = []
    for trading_day in trading_day_list:
        universe_list = config_df[config_df['入池时间'] <= trading_day]['代表产品'].tolist()
        universe = pd.Series(index=universe_list, data=1.).to_frame(trading_day)
        fund_list.append(universe)
    fund_mat = pd.concat(fund_list, axis=1).T.sort_index()
    count_num = fund_mat.sum(axis=1)
    reb_list = count_num[count_num.shift(1) != count_num].index.tolist()
    reb_list.append(end_date)
    # 回测
    ret_list = []
    for i in range(1, len(reb_list)):
        pre_date, t_date = reb_list[i - 1], reb_list[i]
        universe = fund_mat.loc[pre_date].dropna().index.tolist()
        period_data = fund_nav.loc[pre_date: t_date, universe]
        period_data /= period_data.iloc[0]
        period_ret = period_data.mean(axis=1).pct_change().dropna()
        # print(period_data.shape)
        ret_list.append(period_ret)
    ret_df = pd.concat(ret_list).sort_index()
    # 拼接
    trading_day_list = get_trading_day_list(end_date, tday, "week")
    new_formula = get_fund_nav_from_sql(end_date, tday, {"新方程": "SSL554"}).reindex(
        trading_day_list).pct_change().dropna()

    ret_df = pd.concat([ret_df.to_frame('模拟'), new_formula]).sort_index()
    ret_df['拼接'] = np.where(ret_df['新方程'].isnull(), ret_df['模拟'], ret_df['新方程'])
    trading_day_list = get_trading_day_list(start_date, tday, "week")
    benchmark_series = load_benchmark(start_date, tday, '000905').reindex(
            trading_day_list).pct_change().dropna()

    return_df = ret_df[['拼接']].merge(benchmark_series, left_index=True, right_index=True)
    return_df.rename(columns={"拼接": "中小盘二号", "TCLOSE": "中证500"}, inplace=True)
    return_df['超额收益'] = return_df['中小盘二号'] - return_df['中证500']
    nav_df = (1 + return_df).cumprod()
    nav_df.loc[start_date] = 1.
    nav_df = nav_df.sort_index()

    portfolio_index_df = pd.DataFrame(
        index=nav_df.columns, columns=['年化收益', '年化波动', '最大回撤', 'Sharpe', '胜率', '平均损益比'])
    portfolio_index_df.loc[:, '年化收益'] = nav_df.pct_change().dropna(how='all').apply(cal_annual_return, axis=0)
    portfolio_index_df.loc[:, '年化波动'] = \
        nav_df.pct_change().dropna(how='all').apply(cal_annual_volatility, axis=0)
    portfolio_index_df.loc[:, '最大回撤'] = nav_df.apply(cal_max_drawdown, axis=0)
    portfolio_index_df.loc[:, 'Sharpe'] = \
        nav_df.pct_change().dropna(how='all').apply(lambda x: cal_sharpe_ratio(x, 0.015), axis=0)
    portfolio_index_df.loc[:, '胜率'] = \
        nav_df.pct_change().dropna(how='all').apply(lambda x: x.gt(0).sum() / len(x), axis=0)
    portfolio_index_df.loc[:, '平均损益比'] = \
        nav_df.pct_change().dropna(how='all').apply(lambda x: x[x > 0].mean() / x[x < 0].abs().mean(), axis=0)
    portfolio_index_df.index.name = '产品名称'
    portfolio_index_df.reset_index(inplace=True)
    # 格式处理
    portfolio_index_df['年化收益'] = portfolio_index_df['年化收益'].apply(lambda x: format(x, '.2%'))
    portfolio_index_df['年化波动'] = portfolio_index_df['年化波动'].apply(lambda x: format(x, '.2%'))
    portfolio_index_df['最大回撤'] = portfolio_index_df['最大回撤'].apply(lambda x: format(x, '.2%'))
    portfolio_index_df['Sharpe'] = portfolio_index_df['Sharpe'].round(2)
    portfolio_index_df['胜率'] = portfolio_index_df['胜率'].apply(lambda x: format(x, '.1%'))
    portfolio_index_df['平均损益比'] = portfolio_index_df['平均损益比'].round(2)

    return nav_df, portfolio_index_df


def ret_vol_beta(start_date, end_date):
    trading_day_list = get_trading_day_list(start_date, end_date, frequency="week")
    # 超额数据
    alpha_series = pd.read_excel('D:\\alpha_data.xlsx', sheet_name=0)
    alpha_series['TRADEDATE'] = alpha_series['TRADEDATE'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
    alpha_series = alpha_series.set_index('TRADEDATE')[['高频量价']].reindex(trading_day_list).pct_change().dropna()
    # benchmark
    benchmark_series = load_benchmark(start_date, end_date, '000905').reindex(
        trading_day_list).pct_change().dropna()
    # 成交额数据
    sql_script = "SELECT * FROM mac_stock_trading"
    engine = create_engine(engine_params)
    data = pd.read_sql(sql_script, engine)
    data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
    amt_data = data[['trade_date', 'amt_300', 'amt_500', 'amt_1000', 'amt_other']]
    amt_data = amt_data[(amt_data['trade_date'] >= '20201231') & (amt_data['trade_date'] <= '20230217')]
    amt_data = amt_data.set_index('trade_date').sort_index()
    amt_data['amt_all'] = amt_data.sum(axis=1)
    amt_data.loc[trading_day_list, "sign"] = 1.
    amt_data['sign'] = amt_data['sign'].shift(1).fillna(0.).cumsum()
    mean_amt = amt_data.groupby('sign')['amt_all'].mean().to_frame('mean_amt')
    mean_amt['trade_date'] = trading_day_list
    mean_amt = mean_amt.set_index('trade_date')[['mean_amt']]

    df = mean_amt.merge(benchmark_series, left_index=True, right_index=True).merge(
        alpha_series, left_index=True, right_index=True)
    df['mean_amt'] /= 10000.
    df.rename(columns={"TCLOSE": "benchmark", "高频量价": "alpha"}, inplace=True)

    # df = pd.read_excel('D:\\高频数据.xlsx', sheet_name=0, index_col=0)
    # df.rename(columns={"all_std_1": "benchmark", "高频量价": "alpha"}, inplace=True)

    fig = plt.figure()
    ax = fig.add_subplot(111, projection="3d")
    ax.scatter(df[df['alpha'] >= 0]['mean_amt'],
               df[df['alpha'] >= 0]['benchmark'],
               df[df['alpha'] >= 0]['alpha'], c='r')
    ax.scatter(df[df['alpha'] < 0]['mean_amt'],
               df[df['alpha'] < 0]['benchmark'],
               df[df['alpha'] < 0]['alpha'], c='g')

    df.loc[df['mean_amt'] <= df['mean_amt'].quantile(0.33), 'sign_amt'] = 'L'
    df.loc[(df['mean_amt'] > df['mean_amt'].quantile(0.33)) & (df['mean_amt'] <= df['mean_amt'].quantile(0.66)), 'sign_amt'] = 'M'
    df.loc[df['mean_amt'] > df['mean_amt'].quantile(0.66), 'sign_amt'] = 'H'

    df.loc[df['benchmark'] <= df['benchmark'].quantile(0.33), 'sign_benchmark'] = 'L'
    df.loc[(df['benchmark'] > df['benchmark'].quantile(0.33)) & (df['benchmark'] <= df['benchmark'].quantile(0.66)), 'sign_benchmark'] = 'M'
    df.loc[df['benchmark'] > df['benchmark'].quantile(0.66), 'sign_benchmark'] = 'H'

    group_ret = df.groupby(['sign_amt', 'sign_benchmark'])['alpha'].mean().reset_index()
    group_ret = pd.pivot_table(
        group_ret, index='sign_amt', columns='sign_benchmark', values='alpha').loc[["L", "M", "H"]][["L", "M", "H"]]

    return group_ret


if __name__ == '__main__':
    # data_preparation('20130101', '20221230')
    # get_data_from_Wind()
    # curve_fitting("20191227", "20211008", "20230210")
    ret_vol_beta("20201231", "20230217")